var Dom, EventDispatcher, View, addListener, doc, element, elements, emptyEl, getMethods, setMethods, win;
EventDispatcher = require('./EventDispatcher');
View = require('./View');
elements = {
  $win: win = typeof window === 'object' && window,
  $doc: doc = win.document
};
emptyEl = doc && doc.createElement('div');
element = function(id) {
  return elements[id] || (elements[id] = doc.getElementById(id));
};
getMethods = {
  attr: function(el, attr) {
    return el.getAttribute(attr);
  },
  prop: function(el, prop) {
    return el[prop];
  },
  html: function(el) {
    return el.innerHTML;
  }
};
setMethods = {
  attr: function(value, el, attr) {
    return el.setAttribute(attr, value);
  },
  prop: function(value, el, props) {
    var i, last, prop;
    if (Array.isArray(props)) {
      last = props.length - 1;
      i = 0;
      while (i < last) {
        el = el[props[i++]];
      }
      prop = props[last];
    } else {
      prop = props;
    }
    return el[prop] = value;
  },
  propPolite: function(value, el, prop) {
    if (el !== doc.activeElement) {
      return el[prop] = value;
    }
  },
  html: function(value, el, escape) {
    return el.innerHTML = escape ? View.htmlEscape(value) : value;
  },
  appendHtml: function(value, el) {
    var child, _results;
    emptyEl.innerHTML = value;
    _results = [];
    while (child = emptyEl.firstChild) {
      _results.push(el.appendChild(child));
    }
    return _results;
  }
};
addListener = function() {};
Dom = module.exports = function(model, appExports) {
  var events;
  this.events = events = new EventDispatcher({
    onBind: function(name) {
      if (!(name in events._names)) {
        return addListener(name);
      }
    },
    onTrigger: function(name, listener, targetId) {
      var callback, el, fn, id, method, path, property, silent, value;
      if (listener.length === 2) {
        fn = listener[0], id = listener[1];
        if (!(callback = appExports[fn])) {
          return;
        }
      } else {
        fn = listener[0], path = listener[1], id = listener[2], method = listener[3], property = listener[4], silent = listener[5];
      }
      if (id !== targetId) {
        return;
      }
      if (!(el = element(id))) {
        return false;
      }
      if (callback) {
        callback();
        return;
      }
      value = getMethods[method](el, property);
      return (silent ? model.silent : model)[fn](path, value);
    }
  });
};
Dom.prototype = {
  init: function(domEvents) {
    var domHandler, events, name, _results;
    events = this.events;
    domHandler = function(e) {
      var target;
      target = e.target || e.srcElement;
      if (target.nodeType === 3) {
        target = target.parentNode;
      }
      return events.trigger(e.type, target.id);
    };
    if (doc.addEventListener) {
      addListener = function(name) {
        return doc.addEventListener(name, domHandler, false);
      };
    } else if (doc.attachEvent) {
      addListener = function(name) {
        return doc.attachEvent('on' + name, function() {
          return domHandler(event);
        });
      };
    }
    events.set(domEvents);
    _results = [];
    for (name in events._names) {
      _results.push(addListener(name));
    }
    return _results;
  },
  update: function(id, method, property, value) {
    var el;
    if (!(el = element(id))) {
      return false;
    }
    return setMethods[method](value, el, property);
  }
};