/*! Socket.IO.js build:0.7.9, development. Copyright(c) 2011 LearnBoost <dev@learnboost.com> MIT Licensed */

/**
 * socket.io
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

(function (exports) {

  /**
   * IO namespace.
   *
   * @namespace
   */

  var io = exports;

  /**
   * Socket.IO version
   *
   * @api public
   */

  io.version = '0.7.9';

  /**
   * Protocol implemented.
   *
   * @api public
   */

  io.protocol = 1;

  /**
   * Available transports, these will be populated with the available transports
   *
   * @api public
   */

  io.transports = [];

  /**
   * Keep track of jsonp callbacks.
   *
   * @api private
   */

  io.j = [];

  /**
   * Keep track of our io.Sockets
   *
   * @api private
   */
  io.sockets = {};


  /**
   * Manages connections to hosts.
   *
   * @param {String} uri
   * @Param {Boolean} force creation of new socket (defaults to false)
   * @api public
   */

  io.connect = function (host, details) {
    var uri = io.util.parseUri(host)
      , uuri
      , socket;

    if ('undefined' != typeof document) {
      uri.protocol = uri.protocol || document.location.protocol.slice(0, -1);
      uri.host = uri.host || document.domain;
      uri.port = uri.port || document.location.port;
    }

    uuri = io.util.uniqueUri(uri);

    var options = {
        host: uri.host
      , secure: 'https' == uri.protocol
      , port: uri.port || ('https' == uri.protocol ? 443 : 80)
      , query: uri.query || ''
    };

    io.util.merge(options, details);

    if (options['force new connection'] || !io.sockets[uuri]) {
      socket = new io.Socket(options);
    }

    if (!options['force new connection'] && socket) {
      io.sockets[uuri] = socket;
    }

    socket = socket || io.sockets[uuri];

    // if path is different from '' or /
    return socket.of(uri.path.length > 1 ? uri.path : '');
  };

})('object' === typeof module ? module.exports : (window.io = {}));

/**
 * socket.io
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

(function (exports, global) {

  /**
   * Utilities namespace.
   *
   * @namespace
   */

  var util = exports.util = {};

  /**
   * Parses an URI
   *
   * @author Steven Levithan <stevenlevithan.com> (MIT license)
   * @api public
   */

  var re = /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/;

  var parts = ['source', 'protocol', 'authority', 'userInfo', 'user', 'password',
               'host', 'port', 'relative', 'path', 'directory', 'file', 'query',
               'anchor'];

  util.parseUri = function (str) {
    var m = re.exec(str || '')
      , uri = {}
      , i = 14;

    while (i--) {
      uri[parts[i]] = m[i] || '';
    }

    return uri;
  };

  /**
   * Produces a unique url that identifies a Socket.IO connection.
   *
   * @param {Object} uri
   * @api public
   */

  util.uniqueUri = function (uri) {
    var protocol = uri.protocol
      , host = uri.host
      , port = uri.port;

    if ('document' in global) {
      host = host || document.domain;
      port = port || (protocol == 'https'
        && document.location.protocol !== 'https:' ? 443 : document.location.port);
    } else {
      host = host || 'localhost';

      if (!port && protocol == 'https') {
        port = 443;
      }
    }

    return (protocol || 'http') + '://' + host + ':' + (port || 80);
  };

  /**
   * Mergest 2 query strings in to once unique query string
   *
   * @param {String} base
   * @param {String} addition
   * @api public
   */

  util.query = function (base, addition) {
    var query = util.chunkQuery(base || '')
      , components = [];

    util.merge(query, util.chunkQuery(addition || ''));
    for (var part in query) {
      if (query.hasOwnProperty(part)) {
        components.push(part + '=' + query[part]);
      }
    }

    return components.length ? '?' + components.join('&') : '';
  };

  /**
   * Transforms a querystring in to an object
   *
   * @param {String} qs
   * @api public
   */

  util.chunkQuery = function (qs) {
    var query = {}
      , params = qs.split('&')
      , i = 0
      , l = params.length
      , kv;

    for (; i < l; ++i) {
      kv = params[i].split('=');
      if (kv[0]) {
        query[kv[0]] = decodeURIComponent(kv[1]);
      }
    }

    return query;
  };

  /**
   * Executes the given function when the page is loaded.
   *
   *     io.util.load(function () { console.log('page loaded'); });
   *
   * @param {Function} fn
   * @api public
   */

  var pageLoaded = false;

  util.load = function (fn) {
    if ('document' in global && document.readyState === 'complete' || pageLoaded) {
      return fn();
    }

    util.on(global, 'load', fn, false);
  };

  /**
   * Adds an event.
   *
   * @api private
   */

  util.on = function (element, event, fn, capture) {
    if (element.attachEvent) {
      element.attachEvent('on' + event, fn);
    } else if (element.addEventListener) {
      element.addEventListener(event, fn, capture);
    }
  };

  /**
   * Generates the correct `XMLHttpRequest` for regular and cross domain requests.
   *
   * @param {Boolean} [xdomain] Create a request that can be used cross domain.
   * @returns {XMLHttpRequest|false} If we can create a XMLHttpRequest.
   * @api private
   */

  util.request = function (xdomain) {

    if ('undefined' != typeof window) {
      if (xdomain && window.XDomainRequest) {
        return new XDomainRequest();
      }

      if (window.XMLHttpRequest && (!xdomain || util.ua.hasCORS)) {
        return new XMLHttpRequest();
      }

      if (!xdomain) {
        try {
          return new window.ActiveXObject('Microsoft.XMLHTTP');
        } catch(e) { }
      }
    }

    return null;
  };

  /**
   * XHR based transport constructor.
   *
   * @constructor
   * @api public
   */

  /**
   * Change the internal pageLoaded value.
   */

  if ('undefined' != typeof window) {
    util.load(function () {
      pageLoaded = true;
    });
  }

  /**
   * Defers a function to ensure a spinner is not displayed by the browser
   *
   * @param {Function} fn
   * @api public
   */

  util.defer = function (fn) {
    if (!util.ua.webkit) {
      return fn();
    }

    util.load(function () {
      setTimeout(fn, 100);
    });
  };

  /**
   * Merges two objects.
   *
   * @api public
   */
  
  util.merge = function merge (target, additional, deep, lastseen) {
    var seen = lastseen || []
      , depth = typeof deep == 'undefined' ? 2 : deep
      , prop;

    for (prop in additional) {
      if (additional.hasOwnProperty(prop) && util.indexOf(seen, prop) < 0) {
        if (typeof target[prop] !== 'object' || !depth) {
          target[prop] = additional[prop];
          seen.push(additional[prop]);
        } else {
          util.merge(target[prop], additional[prop], depth - 1, seen);
        }
      }
    }

    return target;
  };

  /**
   * Merges prototypes from objects
   *
   * @api public
   */
  
  util.mixin = function (ctor, ctor2) {
    util.merge(ctor.prototype, ctor2.prototype);
  };

  /**
   * Shortcut for prototypical and static inheritance.
   *
   * @api private
   */

  util.inherit = function (ctor, ctor2) {
    function f() {};
    f.prototype = ctor2.prototype;
    ctor.prototype = new f;
  };

  /**
   * Checks if the given object is an Array.
   *
   *     io.util.isArray([]); // true
   *     io.util.isArray({}); // false
   *
   * @param Object obj
   * @api public
   */

  util.isArray = Array.isArray || function (obj) {
    return Object.prototype.toString.call(obj) === '[object Array]';
  };

  /**
   * Intersects values of two arrays into a third
   *
   * @api public
   */

  util.intersect = function (arr, arr2) {
    var ret = []
      , longest = arr.length > arr2.length ? arr : arr2
      , shortest = arr.length > arr2.length ? arr2 : arr;

    for (var i = 0, l = shortest.length; i < l; i++) {
      if (~util.indexOf(longest, shortest[i]))
        ret.push(shortest[i]);
    }

    return ret;
  }

  /**
   * Array indexOf compatibility.
   *
   * @see bit.ly/a5Dxa2
   * @api public
   */

  util.indexOf = function (arr, o, i) {
    if (Array.prototype.indexOf) {
      return Array.prototype.indexOf.call(arr, o, i);
    }

    for (var j = arr.length, i = i < 0 ? i + j < 0 ? 0 : i + j : i || 0; 
         i < j && arr[i] !== o; i++);

    return j <= i ? -1 : i;
  };

  /**
   * Converts enumerables to array.
   *
   * @api public
   */

  util.toArray = function (enu) {
    var arr = [];

    for (var i = 0, l = enu.length; i < l; i++)
      arr.push(enu[i]);

    return arr;
  };

  /**
   * UA / engines detection namespace.
   *
   * @namespace
   */

  util.ua = {};

  /**
   * Whether the UA supports CORS for XHR.
   *
   * @api public
   */

  util.ua.hasCORS = 'undefined' != typeof window && window.XMLHttpRequest &&
  (function () {
    try {
      var a = new XMLHttpRequest();
    } catch (e) {
      return false;
    }

    return a.withCredentials != undefined;
  })();

  /**
   * Detect webkit.
   *
   * @api public
   */

  util.ua.webkit = 'undefined' != typeof navigator
    && /webkit/i.test(navigator.userAgent);

})(
    'undefined' != typeof window ? io : module.exports
  , this
);

/**
 * socket.io
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

(function (exports, io) {

  /**
   * Expose constructor.
   */

  exports.EventEmitter = EventEmitter;

  /**
   * Event emitter constructor.
   *
   * @api public.
   */

  function EventEmitter () {};

  /**
   * Adds a listener
   *
   * @api public
   */

  EventEmitter.prototype.on = function (name, fn) {
    if (!this.$events) {
      this.$events = {};
    }

    if (!this.$events[name]) {
      this.$events[name] = fn;
    } else if (io.util.isArray(this.$events[name])) {
      this.$events[name].push(fn);
    } else {
      this.$events[name] = [this.$events[name], fn];
    }

    return this;
  };

  EventEmitter.prototype.addListener = EventEmitter.prototype.on;

  /**
   * Adds a volatile listener.
   *
   * @api public
   */

  EventEmitter.prototype.once = function (name, fn) {
    var self = this;

    function on () {
      self.removeListener(name, on);
      fn.apply(this, arguments);
    };

    on.listener = fn;
    this.on(name, on);

    return this;
  };

  /**
   * Removes a listener.
   *
   * @api public
   */

  EventEmitter.prototype.removeListener = function (name, fn) {
    if (this.$events && this.$events[name]) {
      var list = this.$events[name];

      if (io.util.isArray(list)) {
        var pos = -1;

        for (var i = 0, l = list.length; i < l; i++) {
          if (list[i] === fn || (list[i].listener && list[i].listener === fn)) {
            pos = i;
            break;
          }
        }

        if (pos < 0) {
          return this;
        }

        list.splice(pos, 1);

        if (!list.length) {
          delete this.$events[name];
        }
      } else if (list === fn || (list.listener && list.listener === fn)) {
        delete this.$events[name];
      }
    }

    return this;
  };

  /**
   * Removes all listeners for an event.
   *
   * @api public
   */

  EventEmitter.prototype.removeAllListeners = function (name) {
    // TODO: enable this when node 0.5 is stable
    //if (name === undefined) {
      //this.$events = {};
      //return this;
    //}

    if (this.$events && this.$events[name]) {
      this.$events[name] = null;
    }

    return this;
  };

  /**
   * Gets all listeners for a certain event.
   *
   * @api publci
   */

  EventEmitter.prototype.listeners = function (name) {
    if (!this.$events) {
      this.$events = {};
    }

    if (!this.$events[name]) {
      this.$events[name] = [];
    }

    if (!io.util.isArray(this.$events[name])) {
      this.$events[name] = [this.$events[name]];
    }

    return this.$events[name];
  };

  /**
   * Emits an event.
   *
   * @api public
   */

  EventEmitter.prototype.emit = function (name) {
    if (!this.$events) {
      return false;
    }

    var handler = this.$events[name];

    if (!handler) {
      return false;
    }

    var args = Array.prototype.slice.call(arguments, 1);

    if ('function' == typeof handler) {
      handler.apply(this, args);
    } else if (io.util.isArray(handler)) {
      var listeners = handler.slice();

      for (var i = 0, l = listeners.length; i < l; i++) {
        listeners[i].apply(this, args);
      }
    } else {
      return false;
    }

    return true;
  };

})(
    'undefined' != typeof io ? io : module.exports
  , 'undefined' != typeof io ? io : module.parent.exports
);

/**
 * socket.io
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

/**
 * Based on JSON2 (http://www.JSON.org/js.html).
 */

(function (exports, nativeJSON) {
  "use strict";

  // use native JSON if it's available
  if (nativeJSON && nativeJSON.parse){
    return exports.JSON = {
      parse: nativeJSON.parse
    , stringify: nativeJSON.stringify
    }
  }

  var JSON = exports.JSON = {};

  function f(n) {
      // Format integers to have at least two digits.
      return n < 10 ? '0' + n : n;
  }

  function date(d, key) {
    return isFinite(d.valueOf()) ?
        d.getUTCFullYear()     + '-' +
        f(d.getUTCMonth() + 1) + '-' +
        f(d.getUTCDate())      + 'T' +
        f(d.getUTCHours())     + ':' +
        f(d.getUTCMinutes())   + ':' +
        f(d.getUTCSeconds())   + 'Z' : null;
  };

  var cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
      escapable = /[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
      gap,
      indent,
      meta = {    // table of character substitutions
          '\b': '\\b',
          '\t': '\\t',
          '\n': '\\n',
          '\f': '\\f',
          '\r': '\\r',
          '"' : '\\"',
          '\\': '\\\\'
      },
      rep;


  function quote(string) {

// If the string contains no control characters, no quote characters, and no
// backslash characters, then we can safely slap some quotes around it.
// Otherwise we must also replace the offending characters with safe escape
// sequences.

      escapable.lastIndex = 0;
      return escapable.test(string) ? '"' + string.replace(escapable, function (a) {
          var c = meta[a];
          return typeof c === 'string' ? c :
              '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
      }) + '"' : '"' + string + '"';
  }


  function str(key, holder) {

// Produce a string from holder[key].

      var i,          // The loop counter.
          k,          // The member key.
          v,          // The member value.
          length,
          mind = gap,
          partial,
          value = holder[key];

// If the value has a toJSON method, call it to obtain a replacement value.

      if (value instanceof Date) {
          value = date(key);
      }

// If we were called with a replacer function, then call the replacer to
// obtain a replacement value.

      if (typeof rep === 'function') {
          value = rep.call(holder, key, value);
      }

// What happens next depends on the value's type.

      switch (typeof value) {
      case 'string':
          return quote(value);

      case 'number':

// JSON numbers must be finite. Encode non-finite numbers as null.

          return isFinite(value) ? String(value) : 'null';

      case 'boolean':
      case 'null':

// If the value is a boolean or null, convert it to a string. Note:
// typeof null does not produce 'null'. The case is included here in
// the remote chance that this gets fixed someday.

          return String(value);

// If the type is 'object', we might be dealing with an object or an array or
// null.

      case 'object':

// Due to a specification blunder in ECMAScript, typeof null is 'object',
// so watch out for that case.

          if (!value) {
              return 'null';
          }

// Make an array to hold the partial results of stringifying this object value.

          gap += indent;
          partial = [];

// Is the value an array?

          if (Object.prototype.toString.apply(value) === '[object Array]') {

// The value is an array. Stringify every element. Use null as a placeholder
// for non-JSON values.

              length = value.length;
              for (i = 0; i < length; i += 1) {
                  partial[i] = str(i, value) || 'null';
              }

// Join all of the elements together, separated with commas, and wrap them in
// brackets.

              v = partial.length === 0 ? '[]' : gap ?
                  '[\n' + gap + partial.join(',\n' + gap) + '\n' + mind + ']' :
                  '[' + partial.join(',') + ']';
              gap = mind;
              return v;
          }

// If the replacer is an array, use it to select the members to be stringified.

          if (rep && typeof rep === 'object') {
              length = rep.length;
              for (i = 0; i < length; i += 1) {
                  if (typeof rep[i] === 'string') {
                      k = rep[i];
                      v = str(k, value);
                      if (v) {
                          partial.push(quote(k) + (gap ? ': ' : ':') + v);
                      }
                  }
              }
          } else {

// Otherwise, iterate through all of the keys in the object.

              for (k in value) {
                  if (Object.prototype.hasOwnProperty.call(value, k)) {
                      v = str(k, value);
                      if (v) {
                          partial.push(quote(k) + (gap ? ': ' : ':') + v);
                      }
                  }
              }
          }

// Join all of the member texts together, separated with commas,
// and wrap them in braces.

          v = partial.length === 0 ? '{}' : gap ?
              '{\n' + gap + partial.join(',\n' + gap) + '\n' + mind + '}' :
              '{' + partial.join(',') + '}';
          gap = mind;
          return v;
      }
  }

// If the JSON object does not yet have a stringify method, give it one.

  JSON.stringify = function (value, replacer, space) {

// The stringify method takes a value and an optional replacer, and an optional
// space parameter, and returns a JSON text. The replacer can be a function
// that can replace values, or an array of strings that will select the keys.
// A default replacer method can be provided. Use of the space parameter can
// produce text that is more easily readable.

      var i;
      gap = '';
      indent = '';

// If the space parameter is a number, make an indent string containing that
// many spaces.

      if (typeof space === 'number') {
          for (i = 0; i < space; i += 1) {
              indent += ' ';
          }

// If the space parameter is a string, it will be used as the indent string.

      } else if (typeof space === 'string') {
          indent = space;
      }

// If there is a replacer, it must be a function or an array.
// Otherwise, throw an error.

      rep = replacer;
      if (replacer && typeof replacer !== 'function' &&
              (typeof replacer !== 'object' ||
              typeof replacer.length !== 'number')) {
          throw new Error('JSON.stringify');
      }

// Make a fake root object containing our value under the key of ''.
// Return the result of stringifying the value.

      return str('', {'': value});
  };

// If the JSON object does not yet have a parse method, give it one.

  JSON.parse = function (text, reviver) {
  // The parse method takes a text and an optional reviver function, and returns
  // a JavaScript value if the text is a valid JSON text.

      var j;

      function walk(holder, key) {

  // The walk method is used to recursively walk the resulting structure so
  // that modifications can be made.

          var k, v, value = holder[key];
          if (value && typeof value === 'object') {
              for (k in value) {
                  if (Object.prototype.hasOwnProperty.call(value, k)) {
                      v = walk(value, k);
                      if (v !== undefined) {
                          value[k] = v;
                      } else {
                          delete value[k];
                      }
                  }
              }
          }
          return reviver.call(holder, key, value);
      }


  // Parsing happens in four stages. In the first stage, we replace certain
  // Unicode characters with escape sequences. JavaScript handles many characters
  // incorrectly, either silently deleting them, or treating them as line endings.

      text = String(text);
      cx.lastIndex = 0;
      if (cx.test(text)) {
          text = text.replace(cx, function (a) {
              return '\\u' +
                  ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
          });
      }

  // In the second stage, we run the text against regular expressions that look
  // for non-JSON patterns. We are especially concerned with '()' and 'new'
  // because they can cause invocation, and '=' because it can cause mutation.
  // But just to be safe, we want to reject all unexpected forms.

  // We split the second stage into 4 regexp operations in order to work around
  // crippling inefficiencies in IE's and Safari's regexp engines. First we
  // replace the JSON backslash pairs with '@' (a non-JSON character). Second, we
  // replace all simple value tokens with ']' characters. Third, we delete all
  // open brackets that follow a colon or comma or that begin the text. Finally,
  // we look to see that the remaining characters are only whitespace or ']' or
  // ',' or ':' or '{' or '}'. If that is so, then the text is safe for eval.

      if (/^[\],:{}\s]*$/
              .test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@')
                  .replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']')
                  .replace(/(?:^|:|,)(?:\s*\[)+/g, ''))) {

  // In the third stage we use the eval function to compile the text into a
  // JavaScript structure. The '{' operator is subject to a syntactic ambiguity
  // in JavaScript: it can begin a block or an object literal. We wrap the text
  // in parens to eliminate the ambiguity.

          j = eval('(' + text + ')');

  // In the optional fourth stage, we recursively walk the new structure, passing
  // each name/value pair to a reviver function for possible transformation.

          return typeof reviver === 'function' ?
              walk({'': j}, '') : j;
      }

  // If the text is not JSON parseable, then a SyntaxError is thrown.

      throw new SyntaxError('JSON.parse');
  };

})(
    'undefined' != typeof io ? io : module.exports
  , typeof JSON !== 'undefined' ? JSON : undefined
);

/**
 * socket.io
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

(function (exports, io) {

  /**
   * Parser namespace.
   *
   * @namespace
   */

  var parser = exports.parser = {};

  /**
   * Packet types.
   */

  var packets = parser.packets = [
      'disconnect'
    , 'connect'
    , 'heartbeat'
    , 'message'
    , 'json'
    , 'event'
    , 'ack'
    , 'error'
    , 'noop'
  ];

  /**
   * Errors reasons.
   */

  var reasons = parser.reasons = [
      'transport not supported'
    , 'client not handshaken'
    , 'unauthorized'
  ];

  /**
   * Errors advice.
   */

  var advice = parser.advice = [
      'reconnect'
  ];

  /**
   * Shortcuts.
   */

  var JSON = io.JSON
    , indexOf = io.util.indexOf;

  /**
   * Encodes a packet.
   *
   * @api private
   */

  parser.encodePacket = function (packet) {
    var type = indexOf(packets, packet.type)
      , id = packet.id || ''
      , endpoint = packet.endpoint || ''
      , ack = packet.ack
      , data = null;

    switch (packet.type) {
      case 'error':
        var reason = packet.reason ? indexOf(reasons, packet.reason) : ''
          , adv = packet.advice ? indexOf(advice, packet.advice) : '';

        if (reason !== '' || adv !== '')
          data = reason + (adv !== '' ? ('+' + adv) : '');

        break;

      case 'message':
        if (packet.data !== '')
          data = packet.data;
        break;

      case 'event':
        var ev = { name: packet.name };

        if (packet.args && packet.args.length) {
          ev.args = packet.args;
        }

        data = JSON.stringify(ev);
        break;

      case 'json':
        data = JSON.stringify(packet.data);
        break;

      case 'connect':
        if (packet.qs)
          data = packet.qs;
        break;

      case 'ack':
        data = packet.ackId
          + (packet.args && packet.args.length
              ? '+' + JSON.stringify(packet.args) : '');
        break;
    }

    // construct packet with required fragments
    var encoded = [
        type
      , id + (ack == 'data' ? '+' : '')
      , endpoint
    ];

    // data fragment is optional
    if (data !== null && data !== undefined)
      encoded.push(data);

    return encoded.join(':');
  };

  /**
   * Encodes multiple messages (payload).
   *
   * @param {Array} messages
   * @api private
   */

  parser.encodePayload = function (packets) {
    var decoded = '';

    if (packets.length == 1)
      return packets[0];

    for (var i = 0, l = packets.length; i < l; i++) {
      var packet = packets[i];
      decoded += '\ufffd' + packet.length + '\ufffd' + packets[i];
    }

    return decoded;
  };

  /**
   * Decodes a packet
   *
   * @api private
   */

  var regexp = /^([^:]+):([0-9]+)?(\+)?:([^:]+)?:?(.*)?$/;

  parser.decodePacket = function (data) {
    var pieces = data.match(regexp);

    if (!pieces) return {};

    var id = pieces[2] || ''
      , data = pieces[5] || ''
      , packet = {
            type: packets[pieces[1]]
          , endpoint: pieces[4] || ''
        };

    // whether we need to acknowledge the packet
    if (id) {
      packet.id = id;
      if (pieces[3])
        packet.ack = 'data';
      else
        packet.ack = true;
    }

    // handle different packet types
    switch (packet.type) {
      case 'error':
        var pieces = data.split('+');
        packet.reason = reasons[pieces[0]] || '';
        packet.advice = advice[pieces[1]] || '';
        break;

      case 'message':
        packet.data = data || '';
        break;

      case 'event':
        try {
          var opts = JSON.parse(data);
          packet.name = opts.name;
          packet.args = opts.args;
        } catch (e) { }

        packet.args = packet.args || [];
        break;

      case 'json':
        try {
          packet.data = JSON.parse(data);
        } catch (e) { }
        break;

      case 'connect':
        packet.qs = data || '';
        break;

      case 'ack':
        var pieces = data.match(/^([0-9]+)(\+)?(.*)/);
        if (pieces) {
          packet.ackId = pieces[1];
          packet.args = [];

          if (pieces[3]) {
            try {
              packet.args = pieces[3] ? JSON.parse(pieces[3]) : [];
            } catch (e) { }
          }
        }
        break;

      case 'disconnect':
      case 'heartbeat':
        break;
    };

    return packet;
  };

  /**
   * Decodes data payload. Detects multiple messages
   *
   * @return {Array} messages
   * @api public
   */

  parser.decodePayload = function (data) {
    // IE doesn't like data[i] for unicode chars, charAt works fine
    if (data.charAt(0) == '\ufffd') {
      var ret = [];

      for (var i = 1, length = ''; i < data.length; i++) {
        if (data.charAt(i) == '\ufffd') {
          ret.push(parser.decodePacket(data.substr(i + 1).substr(0, length)));
          i += Number(length) + 1;
          length = '';
        } else {
          length += data.charAt(i);
        }
      }

      return ret;
    } else {
      return [parser.decodePacket(data)];
    }
  };

})(
    'undefined' != typeof io ? io : module.exports
  , 'undefined' != typeof io ? io : module.parent.exports
);

/**
 * socket.io
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

(function (exports, io) {

  /**
   * Expose constructor.
   */

  exports.Transport = Transport;

  /**
   * This is the transport template for all supported transport methods.
   *
   * @constructor
   * @api public
   */

  function Transport (socket, sessid) {
    this.socket = socket;
    this.sessid = sessid;
  };

  /**
   * Apply EventEmitter mixin.
   */

  io.util.mixin(Transport, io.EventEmitter);

  /**
   * Handles the response from the server. When a new response is received
   * it will automatically update the timeout, decode the message and
   * forwards the response to the onMessage function for further processing.
   *
   * @param {String} data Response from the server.
   * @api private
   */

  Transport.prototype.onData = function (data) {
    this.clearCloseTimeout();
    this.setCloseTimeout();

    if (data !== '') {
      // todo: we should only do decodePayload for xhr transports
      var msgs = io.parser.decodePayload(data);

      if (msgs && msgs.length) {
        for (var i = 0, l = msgs.length; i < l; i++) {
          this.onPacket(msgs[i]);
        }
      }
    }

    return this;
  };

  /**
   * Handles packets.
   *
   * @api private
   */

  Transport.prototype.onPacket = function (packet) {
    if (packet.type == 'heartbeat') {
      return this.onHeartbeat();
    }

    if (packet.type == 'connect' && packet.endpoint == '') {
      this.onConnect();
    }

    this.socket.onPacket(packet);

    return this;
  };

  /**
   * Sets close timeout
   *
   * @api private
   */
  
  Transport.prototype.setCloseTimeout = function () {
    if (!this.closeTimeout) {
      var self = this;

      this.closeTimeout = setTimeout(function () {
        self.onDisconnect();
      }, this.socket.closeTimeout);
    }
  };

  /**
   * Called when transport disconnects.
   *
   * @api private
   */

  Transport.prototype.onDisconnect = function () {
    if (this.close) this.close();
    this.clearTimeouts();
    this.socket.onDisconnect();
    return this;
  };

  /**
   * Called when transport connects
   *
   * @api private
   */

  Transport.prototype.onConnect = function () {
    this.socket.onConnect();
    return this;
  }

  /**
   * Clears close timeout
   *
   * @api private
   */

  Transport.prototype.clearCloseTimeout = function () {
    if (this.closeTimeout) {
      clearTimeout(this.closeTimeout);
      this.closeTimeout = null;
    }
  };

  /**
   * Clear timeouts
   *
   * @api private
   */

  Transport.prototype.clearTimeouts = function () {
    this.clearCloseTimeout();

    if (this.reopenTimeout) {
      clearTimeout(this.reopenTimeout);
    }
  };

  /**
   * Sends a packet
   *
   * @param {Object} packet object.
   * @api private
   */

  Transport.prototype.packet = function (packet) {
    this.send(io.parser.encodePacket(packet));
  };

  /**
   * Send the received heartbeat message back to server. So the server
   * knows we are still connected.
   *
   * @param {String} heartbeat Heartbeat response from the server.
   * @api private
   */

  Transport.prototype.onHeartbeat = function (heartbeat) {
    this.packet({ type: 'heartbeat' });
  };
 
  /**
   * Called when the transport opens.
   *
   * @api private
   */

  Transport.prototype.onOpen = function () {
    this.open = true;
    this.clearCloseTimeout();
    this.socket.onOpen();
  };

  /**
   * Notifies the base when the connection with the Socket.IO server
   * has been disconnected.
   *
   * @api private
   */

  Transport.prototype.onClose = function () {
    var self = this;

    /* FIXME: reopen delay causing a infinit loop
    this.reopenTimeout = setTimeout(function () {
      self.open();
    }, this.socket.options['reopen delay']);*/

    this.open = false;
    this.setCloseTimeout();
    this.socket.onClose();
  };

  /**
   * Generates a connection url based on the Socket.IO URL Protocol.
   * See <https://github.com/learnboost/socket.io-node/> for more details.
   *
   * @returns {String} Connection url
   * @api private
   */

  Transport.prototype.prepareUrl = function () {
    var options = this.socket.options;

    return this.scheme() + '://'
      + options.host + ':' + options.port + '/'
      + options.resource + '/' + io.protocol
      + '/' + this.name + '/' + this.sessid;
  };

  /**
   * Checks if the transport is ready to start a connection.
   *
   * @param {Socket} socket The socket instance that needs a transport
   * @param {Function} fn The callback
   * @api private
   */

  Transport.prototype.ready = function (socket, fn) {
    fn.call(this);
  };
})(
    'undefined' != typeof io ? io : module.exports
  , 'undefined' != typeof io ? io : module.parent.exports
);

/**
 * socket.io
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

(function (exports, io, global) {

  /**
   * Expose constructor.
   */

  exports.Socket = Socket;

  /**
   * Create a new `Socket.IO client` which can establish a persistent
   * connection with a Socket.IO enabled server.
   *
   * @api public
   */

  function Socket (options) {
    this.options = {
        port: 80
      , secure: false
      , document: 'document' in global ? document : false
      , resource: 'socket.io'
      , transports: io.transports
      , 'connect timeout': 10000
      , 'try multiple transports': true
      , 'reconnect': true
      , 'reconnection delay': 500
      , 'reconnection limit': Infinity
      , 'reopen delay': 3000
      , 'max reconnection attempts': 10
      , 'sync disconnect on unload': true
      , 'auto connect': true
    };

    io.util.merge(this.options, options);

    this.connected = false;
    this.open = false;
    this.connecting = false;
    this.reconnecting = false;
    this.namespaces = {};
    this.buffer = [];
    this.doBuffer = false;

    if (this.options['sync disconnect on unload'] &&
        (!this.isXDomain() || io.util.ua.hasCORS)) {
      var self = this;

      io.util.on(global, 'beforeunload', function () {
        self.disconnectSync();
      }, false);
    }

    if (this.options['auto connect']) {
      this.connect();
    }
};

  /**
   * Apply EventEmitter mixin.
   */

  io.util.mixin(Socket, io.EventEmitter);

  /**
   * Returns a namespace listener/emitter for this socket
   *
   * @api public
   */

  Socket.prototype.of = function (name) {
    if (!this.namespaces[name]) {
      this.namespaces[name] = new io.SocketNamespace(this, name);

      if (name !== '') {
        this.namespaces[name].packet({ type: 'connect' });
      }
    }

    return this.namespaces[name];
  };

  /**
   * Emits the given event to the Socket and all namespaces
   *
   * @api private
   */

  Socket.prototype.publish = function () {
    this.emit.apply(this, arguments);

    var nsp;

    for (var i in this.namespaces) {
      if (this.namespaces.hasOwnProperty(i)) {
        nsp = this.of(i);
        nsp.$emit.apply(nsp, arguments);
      }
    }
  };

  /**
   * Performs the handshake
   *
   * @api private
   */

  function empty () { };

  Socket.prototype.handshake = function (fn) {
    var self = this
      , options = this.options;

    function complete (data) {
      if (data instanceof Error) {
        self.onError(data.message);
      } else {
        fn.apply(null, data.split(':'));
      }
    };

    var url = [
          'http' + (options.secure ? 's' : '') + ':/'
        , options.host + ':' + options.port
        , this.options.resource
        , io.protocol
        , io.util.query(this.options.query, 't=' + +new Date)
      ].join('/');

    if (this.isXDomain()) {
      var insertAt = document.getElementsByTagName('script')[0]
        , script = document.createElement('SCRIPT');

      script.src = url + '&jsonp=' + io.j.length;
      insertAt.parentNode.insertBefore(script, insertAt);

      io.j.push(function (data) {
        complete(data);
        script.parentNode.removeChild(script);
      });
    } else {
      var xhr = io.util.request();

      xhr.open('GET', url, true);
      xhr.onreadystatechange = function () {
        if (xhr.readyState == 4) {
          xhr.onreadystatechange = empty;

          if (xhr.status == 200) {
            complete(xhr.responseText);
          } else {
            !self.reconnecting && self.onError(xhr.responseText);
          }
        }
      };
      xhr.send(null);
    }
  };

  /**
   * Find an available transport based on the options supplied in the constructor.
   *
   * @api private
   */

  Socket.prototype.getTransport = function (override) {
    var transports = override || this.transports, match;

    for (var i = 0, transport; transport = transports[i]; i++) {
      if (io.Transport[transport]
        && io.Transport[transport].check(this)
        && (!this.isXDomain() || io.Transport[transport].xdomainCheck())) {
        return new io.Transport[transport](this, this.sessionid);
      }
    }

    return null;
  };

  /**
   * Connects to the server.
   *
   * @param {Function} [fn] Callback.
   * @returns {io.Socket}
   * @api public
   */

  Socket.prototype.connect = function (fn) {
    if (this.connecting) {
      return this;
    }

    var self = this;

    this.handshake(function (sid, heartbeat, close, transports) {
      self.sessionid = sid;
      self.closeTimeout = close * 1000;
      self.heartbeatTimeout = heartbeat * 1000;
      self.transports = io.util.intersect(
          transports.split(',')
        , self.options.transports
      );

      function connect (transports){
        self.transport = self.getTransport(transports);
        if (!self.transport) return self.publish('connect_failed');

        // once the transport is ready
        self.transport.ready(self, function () {
          self.connecting = true;
          self.publish('connecting', self.transport.name);
          self.transport.open();

          if (self.options['connect timeout']) {
            self.connectTimeoutTimer = setTimeout(function () {
              if (!self.connected) {
                self.connecting = false;

                if (self.options['try multiple transports']) {
                  if (!self.remainingTransports) {
                    self.remainingTransports = self.transports.slice(0);
                  }

                  var remaining = self.remainingTransports;

                  while (remaining.length > 0 && remaining.splice(0,1)[0] !=
                         self.transport.name) {}

                    if (remaining.length){
                      connect(remaining);
                    } else {
                      self.publish('connect_failed');
                    }
                }
              }
            }, self.options['connect timeout']);
          }
        });
      }

      connect();

      self.once('connect', function (){
        clearTimeout(self.connectTimeoutTimer);

        fn && typeof fn == 'function' && fn();
      });
    });

    return this;
  };

  /**
   * Sends a message.
   *
   * @param {Object} data packet.
   * @returns {io.Socket}
   * @api public
   */

  Socket.prototype.packet = function (data) {
    if (this.connected && !this.doBuffer) {
      this.transport.packet(data);
    } else {
      this.buffer.push(data);
    }

    return this;
  };

  /**
   * Sets buffer state
   *
   * @api private
   */

  Socket.prototype.setBuffer = function (v) {
    this.doBuffer = v;

    if (!v && this.connected && this.buffer.length) {
      this.transport.payload(this.buffer);
      this.buffer = [];
    }
  };

  /**
   * Disconnect the established connect.
   *
   * @returns {io.Socket}
   * @api public
   */

  Socket.prototype.disconnect = function () {
    if (this.connected) {
      if (this.open) {
        this.of('').packet({ type: 'disconnect' });
      }

      // handle disconnection immediately
      this.onDisconnect('booted');
    }

    return this;
  };

  /**
   * Disconnects the socket with a sync XHR.
   *
   * @api private
   */

  Socket.prototype.disconnectSync = function () {
    // ensure disconnection
    var xhr = io.util.request()
      , uri = this.resource + '/' + io.protocol + '/' + this.sessionid;

    xhr.open('GET', uri, true);

    // handle disconnection immediately
    this.onDisconnect('booted');
  };

  /**
   * Check if we need to use cross domain enabled transports. Cross domain would
   * be a different port or different domain name.
   *
   * @returns {Boolean}
   * @api private
   */

  Socket.prototype.isXDomain = function () {

    var locPort = window.location.port || 80;
    return this.options.host !== document.domain || this.options.port != locPort;
  };

  /**
   * Called upon handshake.
   *
   * @api private
   */

  Socket.prototype.onConnect = function () {
    if (!this.connected) {
      this.connected = true;
      this.connecting = false;
      if (!this.doBuffer) {
        // make sure to flush the buffer
        this.setBuffer(false);
      }
      this.emit('connect');
    }
  };

  /**
   * Called when the transport opens
   *
   * @api private
   */

  Socket.prototype.onOpen = function () {
    this.open = true;
  };

  /**
   * Called when the transport closes.
   *
   * @api private
   */

  Socket.prototype.onClose = function () {
    this.open = false;
  };

  /**
   * Called when the transport first opens a connection
   *
   * @param text
   */

  Socket.prototype.onPacket = function (packet) {
    this.of(packet.endpoint).onPacket(packet);
  };

  /**
   * Handles an error.
   *
   * @api private
   */

  Socket.prototype.onError = function (err) {
    if (err && err.advice) {
      if (err.advice === 'reconnect' && this.connected) {
        this.disconnect();
        this.reconnect();
      }
    }

    this.publish('error', err && err.reason ? err.reason : err);
  };

  /**
   * Called when the transport disconnects.
   *
   * @api private
   */

  Socket.prototype.onDisconnect = function (reason) {
    var wasConnected = this.connected;

    this.connected = false;
    this.connecting = false;
    this.open = false;

    if (wasConnected) {
      this.transport.close();
      this.transport.clearTimeouts();
      this.publish('disconnect', reason);

      if ('booted' != reason && this.options.reconnect && !this.reconnecting) {
        this.reconnect();
      }
    }
  };

  /**
   * Called upon reconnection.
   *
   * @api private
   */

  Socket.prototype.reconnect = function () {
    this.reconnecting = true;
    this.reconnectionAttempts = 0;
    this.reconnectionDelay = this.options['reconnection delay'];

    var self = this
      , maxAttempts = this.options['max reconnection attempts']
      , tryMultiple = this.options['try multiple transports']
      , limit = this.options['reconnection limit'];

    function reset () {
      if (self.connected) {
        for (var i in self.namespaces) {
          if (self.namespaces.hasOwnProperty(i) && '' !== i) {
              self.namespaces[i].packet({ type: 'connect' });
          }
        }
        self.publish('reconnect', self.transport.name, self.reconnectionAttempts);
      }

      self.removeListener('connect_failed', maybeReconnect);
      self.removeListener('connect', maybeReconnect);

      self.reconnecting = false;

      delete self.reconnectionAttempts;
      delete self.reconnectionDelay;
      delete self.reconnectionTimer;
      delete self.redoTransports;

      self.options['try multiple transports'] = tryMultiple;
    };

    function maybeReconnect () {
      if (!self.reconnecting) {
        return;
      }

      if (self.connected) {
        return reset();
      };

      if (self.connecting && self.reconnecting) {
        return self.reconnectionTimer = setTimeout(maybeReconnect, 1000);
      }

      if (self.reconnectionAttempts++ >= maxAttempts) {
        if (!self.redoTransports) {
          self.on('connect_failed', maybeReconnect);
          self.options['try multiple transports'] = true;
          self.transport = self.getTransport();
          self.redoTransports = true;
          self.connect();
        } else {
          self.publish('reconnect_failed');
          reset();
        }
      } else {
        if (self.reconnectionDelay < limit) {
          self.reconnectionDelay *= 2; // exponential back off
        }

        self.connect();
        self.publish('reconnecting', self.reconnectionDelay, self.reconnectionAttempts);
        self.reconnectionTimer = setTimeout(maybeReconnect, self.reconnectionDelay);
      }
    };

    this.options['try multiple transports'] = false;
    this.reconnectionTimer = setTimeout(maybeReconnect, this.reconnectionDelay);

    this.on('connect', maybeReconnect);
  };

})(
    'undefined' != typeof io ? io : module.exports
  , 'undefined' != typeof io ? io : module.parent.exports
  , this
);
/**
 * socket.io
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

(function (exports, io) {

  /**
   * Expose constructor.
   */

  exports.SocketNamespace = SocketNamespace;

  /**
   * Socket namespace constructor.
   *
   * @constructor
   * @api public
   */

  function SocketNamespace (socket, name) {
    this.socket = socket;
    this.name = name || '';
    this.flags = {};
    this.json = new Flag(this, 'json');
    this.ackPackets = 0;
    this.acks = {};
  };

  /**
   * Apply EventEmitter mixin.
   */

  io.util.mixin(SocketNamespace, io.EventEmitter);

  /**
   * Copies emit since we override it
   *
   * @api private
   */

  SocketNamespace.prototype.$emit = io.EventEmitter.prototype.emit;

  /**
   * Creates a new namespace, by proxying the request to the socket. This
   * allows us to use the synax as we do on the server.
   *
   * @api public
   */

  SocketNamespace.prototype.of = function () {
    return this.socket.of.apply(this.socket, arguments);
  };

  /**
   * Sends a packet.
   *
   * @api private
   */

  SocketNamespace.prototype.packet = function (packet) {
    packet.endpoint = this.name;
    this.socket.packet(packet);
    this.flags = {};
    return this;
  };

  /**
   * Sends a message
   *
   * @api public
   */

  SocketNamespace.prototype.send = function (data, fn) {
    var packet = {
        type: this.flags.json ? 'json' : 'message'
      , data: data
    };

    if ('function' == typeof fn) {
      packet.id = ++this.ackPackets;
      packet.ack = true;
      this.acks[packet.id] = fn;
    }

    return this.packet(packet);
  };

  /**
   * Emits an event
   *
   * @api public
   */
  
  SocketNamespace.prototype.emit = function (name) {
    var args = Array.prototype.slice.call(arguments, 1)
      , lastArg = args[args.length - 1]
      , packet = {
            type: 'event'
          , name: name
        };

    if ('function' == typeof lastArg) {
      packet.id = ++this.ackPackets;
      packet.ack = 'data';
      this.acks[packet.id] = lastArg;
      args = args.slice(0, args.length - 1);
    }

    packet.args = args;

    return this.packet(packet);
  };

  /**
   * Disconnects the namespace
   *
   * @api private
   */

  SocketNamespace.prototype.disconnect = function () {
    if (this.name === '') {
      this.socket.disconnect();
    } else {
      this.packet({ type: 'disconnect' });
      this.$emit('disconnect');
    }

    return this;
  };

  /**
   * Handles a packet
   *
   * @api private
   */

  SocketNamespace.prototype.onPacket = function (packet) {
    var self = this;

    function ack () {
      self.packet({
          type: 'ack'
        , args: io.util.toArray(arguments)
        , ackId: packet.id
      });
    };

    switch (packet.type) {
      case 'connect':
        this.$emit('connect');
        break;

      case 'disconnect':
        if (this.name === '') {
          this.socket.onDisconnect(packet.reason || 'booted');
        } else {
          this.$emit('disconnect', packet.reason);
        }
        break;

      case 'message':
      case 'json':
        var params = ['message', packet.data];

        if (packet.ack == 'data') {
          params.push(ack);
        } else if (packet.ack) {
          this.packet({ type: 'ack', ackId: packet.id });
        }

        this.$emit.apply(this, params);
        break;

      case 'event':
        var params = [packet.name].concat(packet.args);

        if (packet.ack == 'data')
          params.push(ack);

        this.$emit.apply(this, params);
        break;

      case 'ack':
        if (this.acks[packet.ackId]) {
          this.acks[packet.ackId].apply(this, packet.args);
          delete this.acks[packet.ackId];
        }
        break;

      case 'error':
        if (packet.advice){
          this.socket.onError(packet);
        } else {
          if (packet.reason == 'unauthorized') {
            this.$emit('connect_failed', packet.reason);
          } else {
            this.$emit('error', packet.reason);
          }
        }
        break;
    }
  };

  /**
   * Flag interface.
   *
   * @api private
   */

  function Flag (nsp, name) {
    this.namespace = nsp;
    this.name = name;
  };

  /**
   * Send a message
   *
   * @api public
   */

  Flag.prototype.send = function () {
    this.namespace.flags[this.name] = true;
    this.namespace.send.apply(this.namespace, arguments);
  };

  /**
   * Emit an event
   *
   * @api public
   */

  Flag.prototype.emit = function () {
    this.namespace.flags[this.name] = true;
    this.namespace.emit.apply(this.namespace, arguments);
  };

})(
    'undefined' != typeof io ? io : module.exports
  , 'undefined' != typeof io ? io : module.parent.exports
);

/**
 * socket.io
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

(function (exports, io) {

  /**
   * Expose constructor.
   */

  exports.websocket = WS;

  /**
   * The WebSocket transport uses the HTML5 WebSocket API to establish an
   * persistent connection with the Socket.IO server. This transport will also
   * be inherited by the FlashSocket fallback as it provides a API compatible
   * polyfill for the WebSockets.
   *
   * @constructor
   * @extends {io.Transport}
   * @api public
   */

  function WS (socket) {
    io.Transport.apply(this, arguments);
  };

  /**
   * Inherits from Transport.
   */

  io.util.inherit(WS, io.Transport);

  /**
   * Transport name
   *
   * @api public
   */

  WS.prototype.name = 'websocket';

  /**
   * Initializes a new `WebSocket` connection with the Socket.IO server. We attach
   * all the appropriate listeners to handle the responses from the server.
   *
   * @returns {Transport}
   * @api public
   */

  WS.prototype.open = function () {
    var self = this
      , query = io.util.query(this.socket.options.query);

    this.websocket = new WebSocket(this.prepareUrl() + query);

    var self = this;
    this.websocket.onopen = function () {
      self.onOpen();
      self.socket.setBuffer(false);
    };
    this.websocket.onmessage = function (ev) {
      self.onData(ev.data);
    };
    this.websocket.onclose = function () {
      self.onClose();
      self.socket.setBuffer(true);
    };
    this.websocket.onerror = function (e) {
      self.onError(e);
    };

    return this;
  };

  /**
   * Send a message to the Socket.IO server. The message will automatically be
   * encoded in the correct message format.
   *
   * @returns {Transport}
   * @api public
   */

  WS.prototype.send = function (data) {
    this.websocket.send(data);
    return this;
  };

  /**
   * Payload
   *
   * @api private
   */

  WS.prototype.payload = function (arr) {
    for (var i = 0, l = arr.length; i < l; i++) {
      this.packet(arr[i]);
    }
    return this;
  };

  /**
   * Disconnect the established `WebSocket` connection.
   *
   * @returns {Transport}
   * @api public
   */

  WS.prototype.close = function () {
    this.websocket.close();
    return this;
  };

  /**
   * Handle the errors that `WebSocket` might be giving when we
   * are attempting to connect or send messages.
   *
   * @param {Error} e The error.
   * @api private
   */

  WS.prototype.onError = function (e) {
    this.socket.onError(e);
  };

  /**
   * Returns the appropriate scheme for the URI generation.
   *
   * @api private
   */
  WS.prototype.scheme = function () {
    return this.socket.options.secure ? 'wss' : 'ws';
  };

  /**
   * Checks if the browser has support for native `WebSockets` and that
   * it's not the polyfill created for the FlashSocket transport.
   *
   * @return {Boolean}
   * @api public
   */

  WS.check = function () {
    return 'WebSocket' in window && !('__addTask' in WebSocket);
  };

  /**
   * Check if the `WebSocket` transport support cross domain communications.
   *
   * @returns {Boolean}
   * @api public
   */

  WS.xdomainCheck = function () {
    return true;
  };

  /**
   * Add the transport to your public io.transports array.
   *
   * @api private
   */

  io.transports.push('websocket');


})(
    'undefined' != typeof io ? io.Transport : module.exports
  , 'undefined' != typeof io ? io : module.parent.exports
);

/**
 * socket.io
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

(function (exports, io, global) {

  /**
   * Expose constructor.
   *
   * @api public
   */
  
  exports.XHR = XHR;

  /**
   * XHR constructor
   *
   * @costructor
   * @api public
   */

  function XHR (socket) {
    if (!socket) return;

    io.Transport.apply(this, arguments);
    this.sendBuffer = [];
  };

  /**
   * Inherits from Transport.
   */

  io.util.inherit(XHR, io.Transport);

  /**
   * Establish a connection
   *
   * @returns {Transport}
   * @api public
   */

  XHR.prototype.open = function () {
    this.socket.setBuffer(false);
    this.onOpen();
    this.get();

    // we need to make sure the request succeeds since we have no indication
    // whether the request opened or not until it succeeded.
    this.setCloseTimeout();

    return this;
  };

  /**
   * Check if we need to send data to the Socket.IO server, if we have data in our
   * buffer we encode it and forward it to the `post` method.
   *
   * @api private
   */

  XHR.prototype.payload = function (payload) {
    var msgs = [];

    for (var i = 0, l = payload.length; i < l; i++) {
      msgs.push(io.parser.encodePacket(payload[i]));
    }

    this.send(io.parser.encodePayload(msgs));
  };

  /**
   * Send data to the Socket.IO server.
   *
   * @param data The message
   * @returns {Transport}
   * @api public
   */

  XHR.prototype.send = function (data) {
    this.post(data);
    return this;
  };

  /**
   * Posts a encoded message to the Socket.IO server.
   *
   * @param {String} data A encoded message.
   * @api private
   */

  function empty () { };

  XHR.prototype.post = function (data) {
    var self = this;
    this.socket.setBuffer(true);

    function stateChange () {
      if (this.readyState == 4) {
        this.onreadystatechange = empty;
        self.posting = false;

        if (this.status == 200){
          self.socket.setBuffer(false);
        } else {
          self.onClose();
        }
      }
    }

    function onload () {
      this.onload = empty;
      self.socket.setBuffer(false);
    };

    this.sendXHR = this.request('POST');

    if (global.XDomainRequest && this.sendXHR instanceof XDomainRequest) {
      this.sendXHR.onload = this.sendXHR.onerror = onload;
    } else {
      this.sendXHR.onreadystatechange = stateChange;
    }

    this.sendXHR.send(data);
  };

  /**
   * Disconnects the established `XHR` connection.
   *
   * @returns {Transport} 
   * @api public
   */

  XHR.prototype.close = function () {
    this.onClose();
    return this;
  };

  /**
   * Generates a configured XHR request
   *
   * @param {String} url The url that needs to be requested.
   * @param {String} method The method the request should use.
   * @returns {XMLHttpRequest}
   * @api private
   */

  XHR.prototype.request = function (method) {
    var req = io.util.request(this.socket.isXDomain())
      , query = io.util.query(this.socket.options.query, 't=' + +new Date);

    req.open(method || 'GET', this.prepareUrl() + query, true);

    if (method == 'POST') {
      try {
        if (req.setRequestHeader) {
          req.setRequestHeader('Content-type', 'text/plain;charset=UTF-8');
        } else {
          // XDomainRequest
          req.contentType = 'text/plain';
        }
      } catch (e) {}
    }

    return req;
  };

  /**
   * Returns the scheme to use for the transport URLs.
   *
   * @api private
   */

  XHR.prototype.scheme = function () {
    return this.socket.options.secure ? 'https' : 'http';
  };

  /**
   * Check if the XHR transports are supported
   *
   * @param {Boolean} xdomain Check if we support cross domain requests.
   * @returns {Boolean}
   * @api public
   */

  XHR.check = function (socket, xdomain) {
    try {
      if (io.util.request(xdomain)) {
        return true;
      }
    } catch(e) {}

    return false;
  };

  /**
   * Check if the XHR transport supports corss domain requests.
   * 
   * @returns {Boolean}
   * @api public
   */

  XHR.xdomainCheck = function () {
    return XHR.check(null, true);
  };

})(
    'undefined' != typeof io ? io.Transport : module.exports
  , 'undefined' != typeof io ? io : module.parent.exports
  , this
);

/**
 * socket.io
 * Copyright(c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

(function (exports, io, global) {

  /**
   * Expose constructor.
   */

  exports['xhr-polling'] = XHRPolling;

  /**
   * The XHR-polling transport uses long polling XHR requests to create a
   * "persistent" connection with the server.
   *
   * @constructor
   * @api public
   */

  function XHRPolling () {
    io.Transport.XHR.apply(this, arguments);
  };

  /**
   * Inherits from XHR transport.
   */

  io.util.inherit(XHRPolling, io.Transport.XHR);

  /**
   * Merge the properties from XHR transport
   */

  io.util.merge(XHRPolling, io.Transport.XHR);

  /**
   * Transport name
   *
   * @api public
   */

  XHRPolling.prototype.name = 'xhr-polling';

  /** 
   * Establish a connection, for iPhone and Android this will be done once the page
   * is loaded.
   *
   * @returns {Transport} Chaining.
   * @api public
   */

  XHRPolling.prototype.open = function () {
    var self = this;

    io.Transport.XHR.prototype.open.call(self);
    return false;
  };

  /**
   * Starts a XHR request to wait for incoming messages.
   *
   * @api private
   */

  function empty () {};

  XHRPolling.prototype.get = function () {
    if (!this.open) return;

    var self = this;

    function stateChange () {
      if (this.readyState == 4) {
        this.onreadystatechange = empty;

        if (this.status == 200) {
          self.onData(this.responseText);
          self.get();
        } else {
          self.onClose();
        }
      }
    };

    function onload () {
      this.onload = empty;
      self.onData(this.responseText);
      self.get();
    };

    this.xhr = this.request();

    if (global.XDomainRequest && this.xhr instanceof XDomainRequest) {
      this.xhr.onload = this.xhr.onerror = onload;
    } else {
      this.xhr.onreadystatechange = stateChange;
    }

    this.xhr.send(null);
  };

  /**
   * Handle the unclean close behavior.
   *
   * @api private
   */

  XHRPolling.prototype.onClose = function () {
    io.Transport.XHR.prototype.onClose.call(this);

    if (this.xhr) {
      this.xhr.onreadystatechange = this.xhr.onload = empty;
      try {
        this.xhr.abort();
      } catch(e){}
      this.xhr = null;
    }
  };

  /**
   * Webkit based browsers show a infinit spinner when you start a XHR request
   * before the browsers onload event is called so we need to defer opening of
   * the transport until the onload event is called. Wrapping the cb in our
   * defer method solve this.
   *
   * @param {Socket} socket The socket instance that needs a transport
   * @param {Function} fn The callback
   * @api private
   */

  XHRPolling.prototype.ready = function (socket, fn) {
    var self = this;

    io.util.defer(function () {
      fn.call(self);
    });
  };

  /**
   * Add the transport to your public io.transports array.
   *
   * @api private
   */

  io.transports.push('xhr-polling');

})(
    'undefined' != typeof io ? io.Transport : module.exports
  , 'undefined' != typeof io ? io : module.parent.exports
  , this
);
;var require = function (file, cwd) {
    var resolved = require.resolve(file, cwd || '/');
    var mod = require.modules[resolved];
    if (!mod) throw new Error(
        'Failed to resolve module ' + file + ', tried ' + resolved
    );
    var res = mod._cached ? mod._cached : mod();
    return res;
}
var __require = require;

require.paths = [];
require.modules = {};
require.extensions = [".js",".coffee"];

require.resolve = (function () {
    var core = {
        'assert': true,
        'events': true,
        'fs': true,
        'path': true,
        'vm': true
    };
    
    return function (x, cwd) {
        if (!cwd) cwd = '/';
        
        if (core[x]) return x;
        var path = require.modules.path();
        var y = cwd || '.';
        
        if (x.match(/^(?:\.\.?\/|\/)/)) {
            var m = loadAsFileSync(path.resolve(y, x))
                || loadAsDirectorySync(path.resolve(y, x));
            if (m) return m;
        }
        
        var n = loadNodeModulesSync(x, y);
        if (n) return n;
        
        throw new Error("Cannot find module '" + x + "'");
        
        function loadAsFileSync (x) {
            if (require.modules[x]) {
                return x;
            }
            
            for (var i = 0; i < require.extensions.length; i++) {
                var ext = require.extensions[i];
                if (require.modules[x + ext]) return x + ext;
            }
        }
        
        function loadAsDirectorySync (x) {
            x = x.replace(/\/+$/, '');
            var pkgfile = x + '/package.json';
            if (require.modules[pkgfile]) {
                var pkg = require.modules[pkgfile]();
                var b = pkg.browserify;
                if (typeof b === 'object' && b.main) {
                    var m = loadAsFileSync(path.resolve(x, b.main));
                    if (m) return m;
                }
                else if (typeof b === 'string') {
                    var m = loadAsFileSync(path.resolve(x, b));
                    if (m) return m;
                }
                else if (pkg.main) {
                    var m = loadAsFileSync(path.resolve(x, pkg.main));
                    if (m) return m;
                }
            }
            
            return loadAsFileSync(x + '/index');
        }
        
        function loadNodeModulesSync (x, start) {
            var dirs = nodeModulesPathsSync(start);
            for (var i = 0; i < dirs.length; i++) {
                var dir = dirs[i];
                var m = loadAsFileSync(dir + '/' + x);
                if (m) return m;
                var n = loadAsDirectorySync(dir + '/' + x);
                if (n) return n;
            }
            
            var m = loadAsFileSync(x);
            if (m) return m;
        }
        
        function nodeModulesPathsSync (start) {
            var parts;
            if (start === '/') parts = [ '' ];
            else parts = path.normalize(start).split('/');
            
            var dirs = [];
            for (var i = parts.length - 1; i >= 0; i--) {
                if (parts[i] === 'node_modules') continue;
                var dir = parts.slice(0, i + 1).join('/') + '/node_modules';
                dirs.push(dir);
            }
            
            return dirs;
        }
    };
})();

require.alias = function (from, to) {
    var path = require.modules.path();
    var res = null;
    try {
        res = require.resolve(from + '/package.json', '/');
    }
    catch (err) {
        res = require.resolve(from, '/');
    }
    var basedir = path.dirname(res);
    
    Object.keys(require.modules)
        .forEach(function (x) {
            if (x.slice(0, basedir.length + 1) === basedir + '/') {
                var f = x.slice(basedir.length);
                require.modules[to + f] = require.modules[basedir + f];
            }
            else if (x === basedir) {
                require.modules[to] = require.modules[basedir];
            }
        })
    ;
};

if (typeof process === 'undefined') process = {};

if (!process.nextTick) process.nextTick = function (fn) {
    setTimeout(fn, 0);
};

if (!process.title) process.title = 'browser';

if (!process.binding) process.binding = function (name) {
    if (name === 'evals') return require('vm')
    else throw new Error('No such module')
};

if (!process.cwd) process.cwd = function () { return '.' };

require.modules["path"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = ".";
    var __filename = "path";
    
    var require = function (file) {
        return __require(file, ".");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, ".");
    };
    
    require.modules = __require.modules;
    __require.modules["path"]._cached = module.exports;
    
    (function () {
        // resolves . and .. elements in a path array with directory names there
// must be no slashes, empty elements, or device names (c:\) in the array
// (so also no leading and trailing slashes - it does not distinguish
// relative and absolute paths)
function normalizeArray(parts, allowAboveRoot) {
  // if the path tries to go above the root, `up` ends up > 0
  var up = 0;
  for (var i = parts.length; i >= 0; i--) {
    var last = parts[i];
    if (last == '.') {
      parts.splice(i, 1);
    } else if (last === '..') {
      parts.splice(i, 1);
      up++;
    } else if (up) {
      parts.splice(i, 1);
      up--;
    }
  }

  // if the path is allowed to go above the root, restore leading ..s
  if (allowAboveRoot) {
    for (; up--; up) {
      parts.unshift('..');
    }
  }

  return parts;
}

// Regex to split a filename into [*, dir, basename, ext]
// posix version
var splitPathRe = /^(.+\/(?!$)|\/)?((?:.+?)?(\.[^.]*)?)$/;

// path.resolve([from ...], to)
// posix version
exports.resolve = function() {
var resolvedPath = '',
    resolvedAbsolute = false;

for (var i = arguments.length; i >= -1 && !resolvedAbsolute; i--) {
  var path = (i >= 0)
      ? arguments[i]
      : process.cwd();

  // Skip empty and invalid entries
  if (typeof path !== 'string' || !path) {
    continue;
  }

  resolvedPath = path + '/' + resolvedPath;
  resolvedAbsolute = path.charAt(0) === '/';
}

// At this point the path should be resolved to a full absolute path, but
// handle relative paths to be safe (might happen when process.cwd() fails)

// Normalize the path
resolvedPath = normalizeArray(resolvedPath.split('/').filter(function(p) {
    return !!p;
  }), !resolvedAbsolute).join('/');

  return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
};

// path.normalize(path)
// posix version
exports.normalize = function(path) {
var isAbsolute = path.charAt(0) === '/',
    trailingSlash = path.slice(-1) === '/';

// Normalize the path
path = normalizeArray(path.split('/').filter(function(p) {
    return !!p;
  }), !isAbsolute).join('/');

  if (!path && !isAbsolute) {
    path = '.';
  }
  if (path && trailingSlash) {
    path += '/';
  }
  
  return (isAbsolute ? '/' : '') + path;
};


// posix version
exports.join = function() {
  var paths = Array.prototype.slice.call(arguments, 0);
  return exports.normalize(paths.filter(function(p, index) {
    return p && typeof p === 'string';
  }).join('/'));
};


exports.dirname = function(path) {
  var dir = splitPathRe.exec(path)[1] || '';
  var isWindows = false;
  if (!dir) {
    // No dirname
    return '.';
  } else if (dir.length === 1 ||
      (isWindows && dir.length <= 3 && dir.charAt(1) === ':')) {
    // It is just a slash or a drive letter with a slash
    return dir;
  } else {
    // It is a full dirname, strip trailing slash
    return dir.substring(0, dir.length - 1);
  }
};


exports.basename = function(path, ext) {
  var f = splitPathRe.exec(path)[2] || '';
  // TODO: make this comparison case-insensitive on windows?
  if (ext && f.substr(-1 * ext.length) === ext) {
    f = f.substr(0, f.length - ext.length);
  }
  return f;
};


exports.extname = function(path) {
  return splitPathRe.exec(path)[3] || '';
};
;
    }).call(module.exports);
    
    __require.modules["path"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/package.json"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer";
    var __filename = "/node_modules/racer/package.json";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/package.json"]._cached = module.exports;
    
    (function () {
        module.exports = {"name":"racer","description":"Experimental realtime model synchronization","version":"0.0.11","main":"./lib/racer.js","browserify":{"main":"./lib/racer.browser.js"},"dependencies":{"redis":">=0.5.11","socket.io":">=0.7","socket.io-client":">=0.7","es5-shim":">=1.2.3","browserify":">=1.4.2","uglify-js":">=1.0.6"},"devDependencies":{"expresso":"https://github.com/bnoguchi/expresso/tarball/627b2c8d1b4d2d1fda2429d0b1b01d00819d9f4a","should":">=0.2.1","coffee-script":">=1.1.1","express":">=2.4.2","mongodb":">=0.9.6-7"},"engines":{"node":">=0.4.0"}};
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/package.json"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/lib/racer.browser.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/lib";
    var __filename = "/node_modules/racer/lib/racer.browser.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/lib/racer.browser.js"]._cached = module.exports;
    
    (function () {
        var Model, isReady, model, racer, util;
require('es5-shim');
util = require('./util');
Model = require('./Model');
io.Socket.prototype.onClose = function() {
  this.open = false;
  this.publish('close');
  return this.onDisconnect();
};
isReady = false;
racer = module.exports = {
  model: model = new Model,
  init: function(options) {
    model._adapter._data = options.data;
    model._adapter.ver = options.base;
    model._clientId = options.clientId;
    model._storeSubs = options.storeSubs;
    model._startId = options.startId;
    model._txnCount = options.txnCount;
    model._onTxnNum(options.txnNum);
    model._setSocket(io.connect(options.ioUri, {
      'reconnection delay': 50,
      'max reconnection attempts': 20
    }));
    isReady = true;
    racer.onready();
    return racer;
  },
  onready: function() {},
  ready: function(onready) {
    return function() {
      var connected;
      racer.onready = onready;
      if (isReady) {
        connected = model.socket.socket.connected;
        onready();
        if (connected) {
          return model.socket.socket.publish('connect');
        }
      }
    };
  },
  util: util
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/lib/racer.browser.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/node_modules/es5-shim/package.json"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/node_modules/es5-shim";
    var __filename = "/node_modules/racer/node_modules/es5-shim/package.json";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/node_modules/es5-shim");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/node_modules/es5-shim");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/node_modules/es5-shim/package.json"]._cached = module.exports;
    
    (function () {
        module.exports = {"name":"es5-shim","version":"1.2.9","description":"ES5 as implementable on previous engines","homepage":"http://github.com/kriskowal/es5-shim/","contributors":["Kris Kowal <kris@cixar.com> (http://github.com/kriskowal/)","Sami Samhuri <sami.samhuri@gmail.com> (http://samhuri.net/)","Florian Schäfer <florian.schaefer@gmail.com> (http://github.com/fschaefer)","Irakli Gozalishvili <rfobic@gmail.com> (http://jeditoolkit.com)","Kit Cambridge <kitcambridge@gmail.com> (http://kitcambridge.github.com)"],"bugs":{"mail":"kris@cixar.com","web":"http://github.com/kriskowal/es5-shim/issues"},"licenses":[{"type":"MIT","url":"http://github.com/kriskowal/es5-shim/raw/master/LICENSE"}],"main":"es5-shim.js","repository":{"type":"git","url":"http://github.com/kriskowal/es5-shim.git"},"engines":{"node":">=0.2.0"}};
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/node_modules/es5-shim/package.json"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/node_modules/es5-shim/es5-shim.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/node_modules/es5-shim";
    var __filename = "/node_modules/racer/node_modules/es5-shim/es5-shim.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/node_modules/es5-shim");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/node_modules/es5-shim");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/node_modules/es5-shim/es5-shim.js"]._cached = module.exports;
    
    (function () {
        // vim:set ts=4 sts=4 sw=4 st:
// -- kriskowal Kris Kowal Copyright (C) 2009-2010 MIT License
// -- tlrobinson Tom Robinson Copyright (C) 2009-2010 MIT License (Narwhal Project)
// -- dantman Daniel Friesen Copyright(C) 2010 XXX No License Specified
// -- fschaefer Florian Schäfer Copyright (C) 2010 MIT License
// -- Irakli Gozalishvili Copyright (C) 2010 MIT License
// -- kitcambridge Kit Cambridge Copyright (C) 2011 MIT License

/*!
    Copyright (c) 2009, 280 North Inc. http://280north.com/
    MIT License. http://github.com/280north/narwhal/blob/master/README.md
*/

(function (definition) {
    // RequireJS
    if (typeof define == "function") {
        define(function () {
            definition();
        });
    // CommonJS and <script>
    } else {
        definition();
    }

})(function (undefined) {

/**
 * Brings an environment as close to ECMAScript 5 compliance
 * as is possible with the facilities of erstwhile engines.
 *
 * ES5 Draft
 * http://www.ecma-international.org/publications/files/drafts/tc39-2009-050.pdf
 *
 * NOTE: this is a draft, and as such, the URL is subject to change.  If the
 * link is broken, check in the parent directory for the latest TC39 PDF.
 * http://www.ecma-international.org/publications/files/drafts/
 *
 * Previous ES5 Draft
 * http://www.ecma-international.org/publications/files/drafts/tc39-2009-025.pdf
 * This is a broken link to the previous draft of ES5 on which most of the
 * numbered specification references and quotes herein were taken.  Updating
 * these references and quotes to reflect the new document would be a welcome
 * volunteer project.
 *
 * @module
 */

/*whatsupdoc*/

//
// Function
// ========
//

// ES-5 15.3.4.5
// http://www.ecma-international.org/publications/files/drafts/tc39-2009-025.pdf

if (!Function.prototype.bind) {
    Function.prototype.bind = function bind(that) { // .length is 1
        // 1. Let Target be the this value.
        var target = this;
        // 2. If IsCallable(Target) is false, throw a TypeError exception.
        // XXX this gets pretty close, for all intents and purposes, letting
        // some duck-types slide
        if (typeof target.apply != "function" || typeof target.call != "function")
            return new TypeError();
        // 3. Let A be a new (possibly empty) internal list of all of the
        //   argument values provided after thisArg (arg1, arg2 etc), in order.
        // XXX slicedArgs will stand in for "A" if used
        var args = slice.call(arguments, 1); // for normal call
        // 4. Let F be a new native ECMAScript object.
        // 9. Set the [[Prototype]] internal property of F to the standard
        //   built-in Function prototype object as specified in 15.3.3.1.
        // 10. Set the [[Call]] internal property of F as described in
        //   15.3.4.5.1.
        // 11. Set the [[Construct]] internal property of F as described in
        //   15.3.4.5.2.
        // 12. Set the [[HasInstance]] internal property of F as described in
        //   15.3.4.5.3.
        // 13. The [[Scope]] internal property of F is unused and need not
        //   exist.
        var bound = function () {

            if (this instanceof bound) {
                // 15.3.4.5.2 [[Construct]]
                // When the [[Construct]] internal method of a function object,
                // F that was created using the bind function is called with a
                // list of arguments ExtraArgs the following steps are taken:
                // 1. Let target be the value of F's [[TargetFunction]]
                //   internal property.
                // 2. If target has no [[Construct]] internal method, a
                //   TypeError exception is thrown.
                // 3. Let boundArgs be the value of F's [[BoundArgs]] internal
                //   property.
                // 4. Let args be a new list containing the same values as the
                //   list boundArgs in the same order followed by the same
                //   values as the list ExtraArgs in the same order.

                var self = Object.create(target.prototype);
                var result = target.apply(
                    self,
                    args.concat(slice.call(arguments))
                );
                if (result !== null && Object(result) === result)
                    return result;
                return self;

            } else {
                // 15.3.4.5.1 [[Call]]
                // When the [[Call]] internal method of a function object, F,
                // which was created using the bind function is called with a
                // this value and a list of arguments ExtraArgs the following
                // steps are taken:
                // 1. Let boundArgs be the value of F's [[BoundArgs]] internal
                //   property.
                // 2. Let boundThis be the value of F's [[BoundThis]] internal
                //   property.
                // 3. Let target be the value of F's [[TargetFunction]] internal
                //   property.
                // 4. Let args be a new list containing the same values as the list
                //   boundArgs in the same order followed by the same values as
                //   the list ExtraArgs in the same order. 5.  Return the
                //   result of calling the [[Call]] internal method of target
                //   providing boundThis as the this value and providing args
                //   as the arguments.

                // equiv: target.call(this, ...boundArgs, ...args)
                return target.apply(
                    that,
                    args.concat(slice.call(arguments))
                );

            }

        };

        // XXX bound.length is never writable, so don't even try
        //
        // 16. The length own property of F is given attributes as specified in
        //   15.3.5.1.
        // TODO
        // 17. Set the [[Extensible]] internal property of F to true.
        // TODO
        // 18. Call the [[DefineOwnProperty]] internal method of F with
        //   arguments "caller", PropertyDescriptor {[[Value]]: null,
        //   [[Writable]]: false, [[Enumerable]]: false, [[Configurable]]:
        //   false}, and false.
        // TODO
        // 19. Call the [[DefineOwnProperty]] internal method of F with
        //   arguments "arguments", PropertyDescriptor {[[Value]]: null,
        //   [[Writable]]: false, [[Enumerable]]: false, [[Configurable]]:
        //   false}, and false.
        // TODO
        // NOTE Function objects created using Function.prototype.bind do not
        // have a prototype property.
        // XXX can't delete it in pure-js.
        return bound;
    };
}

// Shortcut to an often accessed properties, in order to avoid multiple
// dereference that costs universally.
// _Please note: Shortcuts are defined after `Function.prototype.bind` as we
// us it in defining shortcuts.
var call = Function.prototype.call;
var prototypeOfArray = Array.prototype;
var prototypeOfObject = Object.prototype;
var slice = prototypeOfArray.slice;
var toString = prototypeOfObject.toString;
var owns = call.bind(prototypeOfObject.hasOwnProperty);

var defineGetter, defineSetter, lookupGetter, lookupSetter, supportsAccessors;
// If JS engine supports accessors creating shortcuts.
if ((supportsAccessors = owns(prototypeOfObject, '__defineGetter__'))) {
    defineGetter = call.bind(prototypeOfObject.__defineGetter__);
    defineSetter = call.bind(prototypeOfObject.__defineSetter__);
    lookupGetter = call.bind(prototypeOfObject.__lookupGetter__);
    lookupSetter = call.bind(prototypeOfObject.__lookupSetter__);
}

//
// Array
// =====
//

// ES5 15.4.3.2
if (!Array.isArray) {
    Array.isArray = function isArray(obj) {
        return Object.prototype.toString.call(obj) == "[object Array]";
    };
}

// ES5 15.4.4.18
// https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/array/foreach
if (!Array.prototype.forEach) {
    Array.prototype.forEach = function forEach(fun /*, thisp*/) {
        var self = Object(this),
            thisp = arguments[1],
            i = 0,
            length = self.length >>> 0;

        // If no callback function or if callback is not a callable function
        if (!fun || !fun.call) {
            throw new TypeError();
        }

        while (i < length) {
            if (i in self) {
                // Invoke the callback function with call, passing arguments:
                // context, property value, property key, thisArg object context
                fun.call(thisp, self[i], i, self);
            }
            i++;
        }
    };
}

// ES5 15.4.4.19
// https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/map
if (!Array.prototype.map) {
    Array.prototype.map = function map(fun /*, thisp*/) {
        var self = Object(this);
        var length = self.length >>> 0;
        if (typeof fun != "function")
            throw new TypeError();
        var result = new Array(length);
        var thisp = arguments[1];
        for (var i = 0; i < length; i++) {
            if (i in self)
                result[i] = fun.call(thisp, self[i], i, self);
        }
        return result;
    };
}

// ES5 15.4.4.20
if (!Array.prototype.filter) {
    Array.prototype.filter = function filter(fun /*, thisp */) {
        var self = Object(this);
        var length = self.length >>> 0;
        if (typeof fun != "function")
            throw new TypeError();
        var result = [];
        var thisp = arguments[1];
        for (var i = 0; i < length; i++)
            if (i in self && fun.call(thisp, self[i], i, self))
                result.push(self[i]);
        return result;
    };
}

// ES5 15.4.4.16
if (!Array.prototype.every) {
    Array.prototype.every = function every(fun /*, thisp */) {
        if (this === void 0 || this === null)
            throw new TypeError();
        if (typeof fun !== "function")
            throw new TypeError();
        var self = Object(this);
        var length = self.length >>> 0;
        var thisp = arguments[1];
        for (var i = 0; i < length; i++) {
            if (i in self && !fun.call(thisp, self[i], i, self))
                return false;
        }
        return true;
    };
}

// ES5 15.4.4.17
// https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/some
if (!Array.prototype.some) {
    Array.prototype.some = function some(fun /*, thisp */) {
        if (this === void 0 || this === null)
            throw new TypeError();
        if (typeof fun !== "function")
            throw new TypeError();
        var self = Object(this);
        var length = self.length >>> 0;
        var thisp = arguments[1];
        for (var i = 0; i < length; i++) {
            if (i in self && fun.call(thisp, self[i], i, self))
                return true;
        }
        return false;
    };
}

// ES5 15.4.4.21
// https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/reduce
if (!Array.prototype.reduce) {
    Array.prototype.reduce = function reduce(fun /*, initial*/) {
        var self = Object(this);
        var length = self.length >>> 0;
        // Whether to include (... || fun instanceof RegExp)
        // in the following expression to trap cases where
        // the provided function was actually a regular
        // expression literal, which in V8 and
        // JavaScriptCore is a typeof "function".  Only in
        // V8 are regular expression literals permitted as
        // reduce parameters, so it is desirable in the
        // general case for the shim to match the more
        // strict and common behavior of rejecting regular
        // expressions.  However, the only case where the
        // shim is applied is IE's Trident (and perhaps very
        // old revisions of other engines).  In Trident,
        // regular expressions are a typeof "object", so the
        // following guard alone is sufficient.
        if (Object.prototype.toString.call(fun) != "[object Function]")
            throw new TypeError();

        // no value to return if no initial value and an empty array
        if (!length && arguments.length == 1)
            throw new TypeError();

        var i = 0;
        var result;
        if (arguments.length >= 2) {
            result = arguments[1];
        } else {
            do {
                if (i in self) {
                    result = self[i++];
                    break;
                }

                // if array contains no values, no initial value to return
                if (++i >= length)
                    throw new TypeError();
            } while (true);
        }

        for (; i < length; i++) {
            if (i in self)
                result = fun.call(null, result, self[i], i, self);
        }

        return result;
    };
}

// ES5 15.4.4.22
// https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/reduceRight
if (!Array.prototype.reduceRight) {
    Array.prototype.reduceRight = function reduceRight(fun /*, initial*/) {
        var self = Object(this);
        var length = self.length >>> 0;
        if (Object.prototype.toString.call(fun) != "[object Function]")
            throw new TypeError();
        // no value to return if no initial value, empty array
        if (!length && arguments.length == 1)
            throw new TypeError();

        var result, i = length - 1;
        if (arguments.length >= 2) {
            result = arguments[1];
        } else {
            do {
                if (i in self) {
                    result = self[i--];
                    break;
                }

                // if array contains no values, no initial value to return
                if (--i < 0)
                    throw new TypeError();
            } while (true);
        }

        do {
            if (i in this)
                result = fun.call(null, result, self[i], i, self);
        } while (i--);

        return result;
    };
}

// ES5 15.4.4.14
// https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/indexOf
if (!Array.prototype.indexOf) {
    Array.prototype.indexOf = function indexOf(sought /*, fromIndex */ ) {
        if (this === void 0 || this === null)
            throw new TypeError();
        var self = Object(this);
        var length = self.length >>> 0;
        if (!length)
            return -1;
        var i = 0;
        if (arguments.length > 1)
            i = toInteger(arguments[1]);
        // handle negative indicies
        i = i >= 0 ? i : length - Math.abs(i);
        for (; i < length; i++) {
            if (i in self && self[i] === sought) {
                return i;
            }
        }
        return -1;
    }
}

// ES5 15.4.4.15
if (!Array.prototype.lastIndexOf) {
    Array.prototype.lastIndexOf = function lastIndexOf(sought /*, fromIndex */) {
        if (this === void 0 || this === null)
            throw new TypeError();
        var self = Object(this);
        var length = self.length >>> 0;
        if (!length)
            return -1;
        var i = length - 1;
        if (arguments.length > 1)
            i = toInteger(arguments[1]);
        // handle negative indicies
        i = i >= 0 ? i : length - Math.abs(i);
        for (; i >= 0; i--) {
            if (i in self && sought === self[i])
                return i;
        }
        return -1;
    };
}

//
// Object
// ======
//

// ES5 15.2.3.2
if (!Object.getPrototypeOf) {
    // https://github.com/kriskowal/es5-shim/issues#issue/2
    // http://ejohn.org/blog/objectgetprototypeof/
    // recommended by fschaefer on github
    Object.getPrototypeOf = function getPrototypeOf(object) {
        return object.__proto__ || object.constructor.prototype;
        // or undefined if not available in this engine
    };
}

// ES5 15.2.3.3
if (!Object.getOwnPropertyDescriptor) {
    var ERR_NON_OBJECT = "Object.getOwnPropertyDescriptor called on a " +
                         "non-object: ";
    Object.getOwnPropertyDescriptor = function getOwnPropertyDescriptor(object, property) {
        if ((typeof object != "object" && typeof object != "function") || object === null)
            throw new TypeError(ERR_NON_OBJECT + object);
        // If object does not owns property return undefined immediately.
        if (!owns(object, property))
            return undefined;

        var descriptor, getter, setter;

        // If object has a property then it's for sure both `enumerable` and
        // `configurable`.
        descriptor =  { enumerable: true, configurable: true };

        // If JS engine supports accessor properties then property may be a
        // getter or setter.
        if (supportsAccessors) {
            // Unfortunately `__lookupGetter__` will return a getter even
            // if object has own non getter property along with a same named
            // inherited getter. To avoid misbehavior we temporary remove
            // `__proto__` so that `__lookupGetter__` will return getter only
            // if it's owned by an object.
            var prototype = object.__proto__;
            object.__proto__ = prototypeOfObject;

            var getter = lookupGetter(object, property);
            var setter = lookupSetter(object, property);

            // Once we have getter and setter we can put values back.
            object.__proto__ = prototype;

            if (getter || setter) {
                if (getter) descriptor.get = getter;
                if (setter) descriptor.set = setter;

                // If it was accessor property we're done and return here
                // in order to avoid adding `value` to the descriptor.
                return descriptor;
            }
        }

        // If we got this far we know that object has an own property that is
        // not an accessor so we set it as a value and return descriptor.
        descriptor.value = object[property];
        return descriptor;
    };
}

// ES5 15.2.3.4
if (!Object.getOwnPropertyNames) {
    Object.getOwnPropertyNames = function getOwnPropertyNames(object) {
        return Object.keys(object);
    };
}

// ES5 15.2.3.5
if (!Object.create) {
    Object.create = function create(prototype, properties) {
        var object;
        if (prototype === null) {
            object = { "__proto__": null };
        } else {
            if (typeof prototype != "object")
                throw new TypeError("typeof prototype["+(typeof prototype)+"] != 'object'");
            var Type = function () {};
            Type.prototype = prototype;
            object = new Type();
            // IE has no built-in implementation of `Object.getPrototypeOf`
            // neither `__proto__`, but this manually setting `__proto__` will
            // guarantee that `Object.getPrototypeOf` will work as expected with
            // objects created using `Object.create`
            object.__proto__ = prototype;
        }
        if (typeof properties != "undefined")
            Object.defineProperties(object, properties);
        return object;
    };
}

// ES5 15.2.3.6
var oldDefineProperty = Object.defineProperty;
var defineProperty = !!oldDefineProperty;
if (defineProperty) {
    // detect IE 8's DOM-only implementation of defineProperty;
    var subject = {};
    Object.defineProperty(subject, "", {});
    defineProperty = "" in subject;
}
if (!defineProperty) {
    var ERR_NON_OBJECT_DESCRIPTOR = "Property description must be an object: ";
    var ERR_NON_OBJECT_TARGET = "Object.defineProperty called on non-object: "
    var ERR_ACCESSORS_NOT_SUPPORTED = "getters & setters can not be defined " +
                                      "on this javascript engine";

    Object.defineProperty = function defineProperty(object, property, descriptor) {
        if (typeof object != "object" && typeof object != "function")
            throw new TypeError(ERR_NON_OBJECT_TARGET + object);
        if (typeof descriptor != "object" || descriptor === null)
            throw new TypeError(ERR_NON_OBJECT_DESCRIPTOR + descriptor);
        // make a valiant attempt to use the real defineProperty
        // for I8's DOM elements.
        if (oldDefineProperty && object.nodeType)
            return oldDefineProperty(object, property, descriptor);

        // If it's a data property.
        if (owns(descriptor, "value")) {
            // fail silently if "writable", "enumerable", or "configurable"
            // are requested but not supported
            /*
            // alternate approach:
            if ( // can't implement these features; allow false but not true
                !(owns(descriptor, "writable") ? descriptor.writable : true) ||
                !(owns(descriptor, "enumerable") ? descriptor.enumerable : true) ||
                !(owns(descriptor, "configurable") ? descriptor.configurable : true)
            )
                throw new RangeError(
                    "This implementation of Object.defineProperty does not " +
                    "support configurable, enumerable, or writable."
                );
            */

            if (supportsAccessors && (lookupGetter(object, property) ||
                                      lookupSetter(object, property)))
            {
                // As accessors are supported only on engines implementing
                // `__proto__` we can safely override `__proto__` while defining
                // a property to make sure that we don't hit an inherited
                // accessor.
                var prototype = object.__proto__;
                object.__proto__ = prototypeOfObject;
                // Deleting a property anyway since getter / setter may be
                // defined on object itself.
                delete object[property];
                object[property] = descriptor.value;
                // Setting original `__proto__` back now.
                object.__proto__ = prototype;
            } else {
                object[property] = descriptor.value;
            }
        } else {
            if (!supportsAccessors)
                throw new TypeError(ERR_ACCESSORS_NOT_SUPPORTED);
            // If we got that far then getters and setters can be defined !!
            if (owns(descriptor, "get"))
                defineGetter(object, property, descriptor.get);
            if (owns(descriptor, "set"))
                defineSetter(object, property, descriptor.set);
        }

        return object;
    };
}

// ES5 15.2.3.7
if (!Object.defineProperties) {
    Object.defineProperties = function defineProperties(object, properties) {
        for (var property in properties) {
            if (owns(properties, property))
                Object.defineProperty(object, property, properties[property]);
        }
        return object;
    };
}

// ES5 15.2.3.8
if (!Object.seal) {
    Object.seal = function seal(object) {
        // this is misleading and breaks feature-detection, but
        // allows "securable" code to "gracefully" degrade to working
        // but insecure code.
        return object;
    };
}

// ES5 15.2.3.9
if (!Object.freeze) {
    Object.freeze = function freeze(object) {
        // this is misleading and breaks feature-detection, but
        // allows "securable" code to "gracefully" degrade to working
        // but insecure code.
        return object;
    };
}

// detect a Rhino bug and patch it
try {
    Object.freeze(function () {});
} catch (exception) {
    Object.freeze = (function freeze(freezeObject) {
        return function freeze(object) {
            if (typeof object == "function") {
                return object;
            } else {
                return freezeObject(object);
            }
        };
    })(Object.freeze);
}

// ES5 15.2.3.10
if (!Object.preventExtensions) {
    Object.preventExtensions = function preventExtensions(object) {
        // this is misleading and breaks feature-detection, but
        // allows "securable" code to "gracefully" degrade to working
        // but insecure code.
        return object;
    };
}

// ES5 15.2.3.11
if (!Object.isSealed) {
    Object.isSealed = function isSealed(object) {
        return false;
    };
}

// ES5 15.2.3.12
if (!Object.isFrozen) {
    Object.isFrozen = function isFrozen(object) {
        return false;
    };
}

// ES5 15.2.3.13
if (!Object.isExtensible) {
    Object.isExtensible = function isExtensible(object) {
        return true;
    };
}

// ES5 15.2.3.14
// http://whattheheadsaid.com/2010/10/a-safer-object-keys-compatibility-implementation
if (!Object.keys) {

    var hasDontEnumBug = true,
        dontEnums = [
            'toString',
            'toLocaleString',
            'valueOf',
            'hasOwnProperty',
            'isPrototypeOf',
            'propertyIsEnumerable',
            'constructor'
        ],
        dontEnumsLength = dontEnums.length;

    for (var key in {"toString": null})
        hasDontEnumBug = false;

    Object.keys = function keys(object) {

        if (
            typeof object != "object" && typeof object != "function"
            || object === null
        )
            throw new TypeError("Object.keys called on a non-object");

        var keys = [];
        for (var name in object) {
            if (owns(object, name)) {
                keys.push(name);
            }
        }

        if (hasDontEnumBug) {
            for (var i = 0, ii = dontEnumsLength; i < ii; i++) {
                var dontEnum = dontEnums[i];
                if (owns(object, dontEnum)) {
                    keys.push(dontEnum);
                }
            }
        }

        return keys;
    };

}

//
// Date
// ====
//

// ES5 15.9.5.43
// Format a Date object as a string according to a simplified subset of the ISO 8601
// standard as defined in 15.9.1.15.
if (!Date.prototype.toISOString) {
    Date.prototype.toISOString = function toISOString() {
        var result, length, value;
        if (!isFinite(this))
            throw new RangeError;

        // the date time string format is specified in 15.9.1.15.
        result = [this.getUTCFullYear(), this.getUTCMonth() + 1, this.getUTCDate(),
            this.getUTCHours(), this.getUTCMinutes(), this.getUTCSeconds()];

        length = result.length;
        while (length--) {
            value = result[length];
            // pad months, days, hours, minutes, and seconds to have two digits.
            if (value < 10)
                result[length] = '0' + value;
        }
        // pad milliseconds to have three digits.
        return result.slice(0, 3).join('-') + 'T' + result.slice(3).join(':') + '.' +
            ('000' + this.getUTCMilliseconds()).slice(-3) + 'Z';
    }
}

// ES5 15.9.4.4
if (!Date.now) {
    Date.now = function now() {
        return new Date().getTime();
    };
}

// ES5 15.9.5.44
if (!Date.prototype.toJSON) {
    Date.prototype.toJSON = function toJSON(key) {
        // This function provides a String representation of a Date object for
        // use by JSON.stringify (15.12.3). When the toJSON method is called
        // with argument key, the following steps are taken:

        // 1.  Let O be the result of calling ToObject, giving it the this
        // value as its argument.
        // 2. Let tv be ToPrimitive(O, hint Number).
        // 3. If tv is a Number and is not finite, return null.
        // XXX
        // 4. Let toISO be the result of calling the [[Get]] internal method of
        // O with argument "toISOString".
        // 5. If IsCallable(toISO) is false, throw a TypeError exception.
        // XXX this gets pretty close, for all intents and purposes, letting
        // some duck-types slide
        if (typeof this.toISOString.call != "function")
            throw new TypeError();
        // 6. Return the result of calling the [[Call]] internal method of
        // toISO with O as the this value and an empty argument list.
        return this.toISOString.call(this);

        // NOTE 1 The argument is ignored.

        // NOTE 2 The toJSON function is intentionally generic; it does not
        // require that its this value be a Date object. Therefore, it can be
        // transferred to other kinds of objects for use as a method. However,
        // it does require that any such object have a toISOString method. An
        // object is free to use the argument key to filter its
        // stringification.
    };
}

// 15.9.4.2 Date.parse (string)
// 15.9.1.15 Date Time String Format
// Date.parse
// based on work shared by Daniel Friesen (dantman)
// http://gist.github.com/303249
if (isNaN(Date.parse("2011-06-15T21:40:05+06:00"))) {
    // XXX global assignment won't work in embeddings that use
    // an alternate object for the context.
    Date = (function(NativeDate) {

        // Date.length === 7
        var Date = function(Y, M, D, h, m, s, ms) {
            var length = arguments.length;
            if (this instanceof NativeDate) {
                var date = length == 1 && String(Y) === Y ? // isString(Y)
                    // We explicitly pass it through parse:
                    new NativeDate(Date.parse(Y)) :
                    // We have to manually make calls depending on argument
                    // length here
                    length >= 7 ? new NativeDate(Y, M, D, h, m, s, ms) :
                    length >= 6 ? new NativeDate(Y, M, D, h, m, s) :
                    length >= 5 ? new NativeDate(Y, M, D, h, m) :
                    length >= 4 ? new NativeDate(Y, M, D, h) :
                    length >= 3 ? new NativeDate(Y, M, D) :
                    length >= 2 ? new NativeDate(Y, M) :
                    length >= 1 ? new NativeDate(Y) :
                                  new NativeDate();
                // Prevent mixups with unfixed Date object
                date.constructor = Date;
                return date;
            }
            return NativeDate.apply(this, arguments);
        };

        // 15.9.1.15 Date Time String Format. This pattern does not implement
        // extended years ((15.9.1.15.1), as `Date.UTC` cannot parse them.
        var isoDateExpression = new RegExp("^" +
            "(\d{4})" + // four-digit year capture
            "(?:-(\d{2})" + // optional month capture
            "(?:-(\d{2})" + // optional day capture
            "(?:" + // capture hours:minutes:seconds.milliseconds
                "T(\d{2})" + // hours capture
                ":(\d{2})" + // minutes capture
                "(?:" + // optional :seconds.milliseconds
                    ":(\d{2})" + // seconds capture
                    "(?:\.(\d{3}))?" + // milliseconds capture
                ")?" +
            "(?:" + // capture UTC offset component
                "Z|" + // UTC capture
                "(?:" + // offset specifier +/-hours:minutes
                    "([-+])" + // sign capture
                    "(\d{2})" + // hours offset capture
                    ":(\d{2})" + // minutes offest capture
                ")" +
            ")?)?)?)?" +
        "$");

        // Copy any custom methods a 3rd party library may have added
        for (var key in NativeDate)
            Date[key] = NativeDate[key];

        // Copy "native" methods explicitly; they may be non-enumerable
        Date.now = NativeDate.now;
        Date.UTC = NativeDate.UTC;
        Date.prototype = NativeDate.prototype;
        Date.prototype.constructor = Date;

        // Upgrade Date.parse to handle simplified ISO 8601 strings
        Date.parse = function parse(string) {
            var match = isoDateExpression.exec(string);
            if (match) {
                match.shift(); // kill match[0], the full match
                // parse months, days, hours, minutes, seconds, and milliseconds
                for (var i = 1; i < 7; i++) {
                    // provide default values if necessary
                    match[i] = +(match[i] || (i < 3 ? 1 : 0));
                    // match[1] is the month. Months are 0-11 in JavaScript
                    // `Date` objects, but 1-12 in ISO notation, so we
                    // decrement.
                    if (i == 1)
                        match[i]--;
                }

                // parse the UTC offset component
                var minutesOffset = +match.pop(), hourOffset = +match.pop(), sign = match.pop();

                // compute the explicit time zone offset if specified
                var offset = 0;
                if (sign) {
                    // detect invalid offsets and return early
                    if (hourOffset > 23 || minuteOffset > 59)
                        return NaN;

                    // express the provided time zone offset in minutes. The offset is
                    // negative for time zones west of UTC; positive otherwise.
                    offset = (hourOffset * 60 + minuteOffset) * 6e4 * (sign == "+" ? -1 : 1);
                }

                // compute a new UTC date value, accounting for the optional offset
                return NativeDate.UTC.apply(this, match) + offset;
            }
            return NativeDate.parse.apply(this, arguments);
        };

        return Date;
    })(Date);
}

//
// String
// ======
//

// ES5 15.5.4.20
if (!String.prototype.trim) {
    // http://blog.stevenlevithan.com/archives/faster-trim-javascript
    // http://perfectionkills.com/whitespace-deviations/
    var s = "[\x09\x0A\-\x0D\x20\xA0\u1680\u180E\u2000-\u200A\u202F" +
        "\u205F\u3000\u2028\u2029\uFEFF]"
    var trimBeginRegexp = new RegExp("^" + s + s + "*");
    var trimEndRegexp = new RegExp(s + s + "*$");
    String.prototype.trim = function trim() {
        return String(this).replace(trimBeginRegexp, "").replace(trimEndRegexp, "");
    };
}

//
// Util
// ======
//

// http://jsperf.com/to-integer
var toInteger = function (n) {
    n = +n;
    if (n !== n) // isNaN
        n = -1;
    else if (n !== 0 && n !== (1/0) && n !== -(1/0))
        n = (n > 0 || -1) * Math.floor(Math.abs(n));
    return n;
};

});
;
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/node_modules/es5-shim/es5-shim.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/lib/util/index.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/lib/util";
    var __filename = "/node_modules/racer/lib/util/index.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/lib/util");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/lib/util");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/lib/util/index.js"]._cached = module.exports;
    
    (function () {
        module.exports = {
  merge: function(a, b) {
    var k, v;
    for (k in b) {
      v = b[k];
      a[k] = v;
    }
    return a;
  },
  hasKeys: function(o, options) {
    var ignore, k;
    if (options == null) {
      options = {};
    }
    ignore = options.ignore;
    for (k in o) {
      if (ignore && -1 !== ignore.indexOf(k)) {
        continue;
      }
      return true;
    }
    return false;
  }
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/lib/util/index.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/lib/Model.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/lib";
    var __filename = "/node_modules/racer/lib/Model.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/lib/Model.js"]._cached = module.exports;
    
    (function () {
        var EventEmitter, MemorySync, Model, RESEND_INTERVAL, RefHelper, SEND_TIMEOUT, TxnApplier, arrayMethod, merge, pathParser, specHelper, transaction;
var __slice = Array.prototype.slice;
transaction = require('./transaction');
pathParser = require('./pathParser');
MemorySync = require('./adapters/MemorySync');
TxnApplier = require('./TxnApplier');
RefHelper = require('./RefHelper');
specHelper = require('./specHelper');
EventEmitter = require('events').EventEmitter;
merge = require('./util').merge;
Model = module.exports = function(_clientId, AdapterClass) {
  var adapter, method, refHelper, self, txnApplier, txnQueue, txns, _fn, _i, _len, _ref;
  this._clientId = _clientId != null ? _clientId : '';
  if (AdapterClass == null) {
    AdapterClass = MemorySync;
  }
  self = this;
  self._adapter = adapter = new AdapterClass;
  self._cache = {
    invalidateSpecModelCache: function() {
      delete this.obj;
      delete this.lastReplayedTxnId;
      return delete this.path;
    }
  };
  self._storeSubs = [];
  self._startId = '';
  self._txnCount = 0;
  self._txns = txns = {};
  self._txnQueue = txnQueue = [];
  txnApplier = new TxnApplier({
    applyTxn: function(txn) {
      if (transaction.base(txn) > adapter.ver) {
        return self._applyTxn(txn, !txn.emitted && this._clientId !== transaction.clientId(txn));
      }
    },
    onTimeout: function() {
      return self._reqNewTxns();
    }
  });
  self._onTxn = function(txn, num) {
    var queuedTxn;
    if (queuedTxn = txns[transaction.id(txn)]) {
      txn.callback = queuedTxn.callback;
      txn.emitted = queuedTxn.emitted;
    }
    return txnApplier.add(txn, num);
  };
  self._onTxnNum = function(num) {
    txnApplier.setIndex((+num || 0) + 1);
    return txnApplier.clearPending();
  };
  self._removeTxn = function(txnId) {
    var i;
    delete txns[txnId];
    if (~(i = txnQueue.indexOf(txnId))) {
      txnQueue.splice(i, 1);
    }
    return self._cache.invalidateSpecModelCache();
  };
  self.force = Object.create(self, {
    _force: {
      value: true
    }
  });
  self.silent = Object.create(self, {
    _silent: {
      value: true
    }
  });
  this._refHelper = refHelper = new RefHelper(this);
  _ref = ['set', 'del', 'push', 'pop', 'insertAfter', 'insertBefore', 'remove', 'splice', 'move'];
  _fn = function(method) {
    return self.on(method, function(_arg) {
      var args, path;
      path = _arg[0], args = 2 <= _arg.length ? __slice.call(_arg, 1) : [];
      return refHelper.notifyPointersTo(path, this.get(), method, args);
    });
  };
  for (_i = 0, _len = _ref.length; _i < _len; _i++) {
    method = _ref[_i];
    _fn(method);
  }
};
Model.prototype = {
  _commit: function() {},
  _reqNewTxns: function() {},
  _setSocket: function(socket) {
    var adapter, clientId, commit, onConnected, onTxn, removeTxn, resend, resendInterval, self, storeSubs, txnQueue, txns;
    this.socket = socket;
    self = this;
    adapter = this._adapter;
    txns = this._txns;
    txnQueue = this._txnQueue;
    onTxn = this._onTxn;
    removeTxn = this._removeTxn;
    this._commit = commit = function(txn) {
      if (!socket.socket.connected) {
        return;
      }
      txn.timeout = +new Date + SEND_TIMEOUT;
      return socket.emit('txn', txn, self._startId);
    };
    this._reqNewTxns = function() {
      return socket.emit('txnsSince', adapter.ver + 1, self._startId);
    };
    socket.on('txn', onTxn);
    socket.on('txnNum', this._onTxnNum);
    socket.on('txnOk', function(txnId, base, num) {
      var txn;
      if (txn = txns[txnId]) {
        transaction.base(txn, base);
        return onTxn(txn, num);
      }
    });
    socket.on('txnErr', function(err, txnId) {
      var args, callback, txn;
      txn = txns[txnId];
      if (txn && (callback = txn.callback)) {
        args = transaction.args(txn).slice(0);
        args.unshift(err);
        callback.apply(null, args);
      }
      return removeTxn(txnId);
    });
    this.canConnect = true;
    socket.on('fatalErr', function() {
      self.canConnect = false;
      self.emit('canConnect', false);
      return socket.disconnect();
    });
    this.connected = false;
    onConnected = function() {
      self.emit('connected', self.connected);
      return self.emit('connectionStatus', self.connected, self.canConnect);
    };
    clientId = this._clientId;
    storeSubs = this._storeSubs;
    resendInterval = null;
    resend = function() {
      var id, now, txn, _i, _len, _results;
      now = +new Date;
      _results = [];
      for (_i = 0, _len = txnQueue.length; _i < _len; _i++) {
        id = txnQueue[_i];
        txn = txns[id];
        if (txn.timeout > now) {
          return;
        }
        _results.push(commit(txn));
      }
      return _results;
    };
    socket.on('connect', function() {
      var id, _i, _len;
      self.connected = true;
      onConnected();
      socket.emit('sub', clientId, storeSubs, adapter.ver, self._startId);
      for (_i = 0, _len = txnQueue.length; _i < _len; _i++) {
        id = txnQueue[_i];
        commit(txns[id]);
      }
      if (!resendInterval) {
        return resendInterval = setInterval(resend, RESEND_INTERVAL);
      }
    });
    socket.on('disconnect', function() {
      self.connected = false;
      if (resendInterval) {
        clearInterval(resendInterval);
      }
      resendInterval = null;
      return setTimeout(onConnected, 200);
    });
    return socket.on('connect_failed', onConnected);
  },
  _nextTxnId: function() {
    return this._clientId + '.' + this._txnCount++;
  },
  _addTxn: function() {
    var $k, $r, args, callback, firstArgs, id, idAsIndex, members, meta, method, model, path, refHelper, txn, txnArgs, ver, _i, _ref, _ref2;
    method = arguments[0], path = arguments[1], args = 4 <= arguments.length ? __slice.call(arguments, 2, _i = arguments.length - 1) : (_i = 2, []), callback = arguments[_i++];
    refHelper = this._refHelper;
    model = this;
    if (_ref = refHelper.isArrayRef(path, this._specModel()[0]), $r = _ref.$r, $k = _ref.$k, _ref) {
      _ref2 = refHelper.splitArrayArgs(method, args), firstArgs = _ref2[0], members = _ref2[1];
      members = members.map(function(member) {
        if (refHelper.isRefObj(member)) {
          return member;
        }
        model.set($r + '.' + member.id, member);
        return {
          $r: $r,
          $k: member.id.toString()
        };
      });
      args = firstArgs.concat(members);
    }
    if (arrayMethod[method]) {
      idAsIndex = refHelper.arrRefIndex(args[0], path, this._specModel()[0]);
    }
    ver = this._force ? null : this._adapter.ver;
    id = this._nextTxnId();
    txn = transaction.create({
      base: ver,
      id: id,
      method: method,
      args: [path].concat(__slice.call(args))
    });
    txn = refHelper.dereferenceTxn(txn, this._specModel()[0]);
    this._txns[id] = txn;
    txn.callback = callback;
    this._txnQueue.push(id);
    txnArgs = transaction.args(txn);
    path = txnArgs[0];
    if (pathParser.isPrivate(path)) {
      this._cache.invalidateSpecModelCache();
      return this._applyTxn(txn, !txn.emitted && !this._silent);
    }
    if (idAsIndex !== void 0) {
      meta = txnArgs[1];
      meta.index = idAsIndex;
      transaction.meta(txn, meta);
    }
    if (!this._silent) {
      this.emit(method, txnArgs);
      txn.emitted = true;
    }
    if (idAsIndex !== void 0) {
      txnArgs[1] = idAsIndex;
    }
    return this._commit(txn);
  },
  _applyTxn: function(txn, forceEmit) {
    var args, callback, meta, method, txnArgs, _ref;
    method = transaction.method(txn);
    txnArgs = transaction.args(txn);
    args = txnArgs.slice(0);
    args.push(transaction.base(txn));
    (_ref = this._adapter)[method].apply(_ref, args);
    this._removeTxn(transaction.id(txn));
    if (forceEmit) {
      if (meta = transaction.meta(txn)) {
        args[1] = meta;
      }
      this.emit(method, args);
    }
    if (callback = txn.callback) {
      return callback.apply(null, [null].concat(__slice.call(txnArgs)));
    }
  },
  _specModel: function() {
    var adapter, args, cache, i, lastReplayedTxnId, len, meta, obj, path, replayFrom, txn;
    cache = this._cache;
    if (lastReplayedTxnId = cache.lastReplayedTxnId) {
      if (cache.lastReplayedTxnId === this._txnQueue[this._txnQueue.length - 1]) {
        return [cache.obj, cache.path];
      }
      obj = cache.obj;
      replayFrom = 1 + this._txnQueue.indexOf(cache.lastReplayedTxnId);
    } else {
      replayFrom = 0;
    }
    adapter = this._adapter;
    if (len = this._txnQueue.length) {
      if (!obj) {
        obj = cache.obj = specHelper.create(adapter._data);
      }
      i = replayFrom;
      while (i < len) {
        txn = this._txns[this._txnQueue[i++]];
        args = transaction.args(txn).slice(0);
        args.push(adapter.ver, {
          obj: obj,
          proto: true,
          returnMeta: true
        });
        meta = adapter[transaction.method(txn)].apply(adapter, args);
        path = meta.path;
      }
      cache.obj = obj;
      cache.path = path;
      cache.lastReplayedTxnId = transaction.id(txn);
    }
    return [obj, path];
  },
  ref: RefHelper.prototype.ref,
  arrayRef: RefHelper.prototype.arrayRef,
  get: function(path) {
    return this._adapter.get(path, this._specModel()[0]);
  },
  set: function(path, value, callback) {
    this._addTxn('set', path, value, callback);
    return value;
  },
  setNull: function(path, value, callback) {
    var obj;
    obj = this.get(path);
    if (obj != null) {
      return obj;
    }
    return this.set(path, value, callback);
  },
  del: function(path, callback) {
    return this._addTxn('del', path, callback);
  },
  incr: function(path, byNum, callback) {
    if (typeof byNum === 'function') {
      callback = byNum;
      byNum = 1;
    } else if (typeof byNum !== 'number') {
      byNum = 1;
    }
    return this.set(path, (this.get(path) || 0) + byNum, callback);
  },
  push: function() {
    var callback, path, values, _i;
    path = arguments[0], values = 3 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 1) : (_i = 1, []), callback = arguments[_i++];
    if ('function' !== typeof callback && callback !== void 0) {
      values.push(callback);
      callback = null;
    }
    return this._addTxn.apply(this, ['push', path].concat(__slice.call(values), [callback]));
  },
  pop: function(path, callback) {
    return this._addTxn('pop', path, callback);
  },
  unshift: function() {
    var callback, path, values, _i;
    path = arguments[0], values = 3 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 1) : (_i = 1, []), callback = arguments[_i++];
    if ('function' !== typeof callback && callback !== void 0) {
      values.push(callback);
      callback = null;
    }
    return this._addTxn.apply(this, ['unshift', path].concat(__slice.call(values), [callback]));
  },
  shift: function(path, callback) {
    return this._addTxn('shift', path, callback);
  },
  insertAfter: function(path, afterIndex, value, callback) {
    return this._addTxn('insertAfter', path, afterIndex, value, callback);
  },
  insertBefore: function(path, beforeIndex, value, callback) {
    return this._addTxn('insertBefore', path, beforeIndex, value, callback);
  },
  remove: function(path, start, howMany, callback) {
    if (howMany == null) {
      howMany = 1;
    }
    if (typeof howMany === 'function') {
      callback = howMany;
      howMany = 1;
    }
    return this._addTxn('remove', path, start, howMany, callback);
  },
  splice: function() {
    var callback, newMembers, path, removeCount, startIndex, _i;
    path = arguments[0], startIndex = arguments[1], removeCount = arguments[2], newMembers = 5 <= arguments.length ? __slice.call(arguments, 3, _i = arguments.length - 1) : (_i = 3, []), callback = arguments[_i++];
    if ('function' !== typeof callback && callback !== void 0) {
      newMembers.push(callback);
      callback = null;
    }
    return this._addTxn.apply(this, ['splice', path, startIndex, removeCount].concat(__slice.call(newMembers), [callback]));
  },
  move: function(path, from, to, callback) {
    return this._addTxn('move', path, from, to, callback);
  }
};
Model._SEND_TIMEOUT = SEND_TIMEOUT = 10000;
Model._RESEND_INTERVAL = RESEND_INTERVAL = 2000;
arrayMethod = {
  remove: 1,
  insertAfter: 1,
  insertBefore: 1,
  splice: 1,
  move: 1
};
merge(Model.prototype, EventEmitter.prototype);
Model.prototype._eventListener = function(method, pattern, callback) {
  var re;
  if (pattern.call) {
    return pattern;
  }
  re = pathParser.regExp(pattern);
  return function(_arg) {
    var args, path;
    path = _arg[0], args = 2 <= _arg.length ? __slice.call(_arg, 1) : [];
    if (re.test(path)) {
      return callback.apply(null, re.exec(path).slice(1).concat(args));
    }
  };
};
Model.prototype._on = EventEmitter.prototype.on;
Model.prototype.on = Model.prototype.addListener = function(type, pattern, callback) {
  var listener;
  this._on(type, listener = this._eventListener(type, pattern, callback));
  return listener;
};
Model.prototype._once = EventEmitter.prototype.once;
Model.prototype.once = function(type, pattern, callback) {
  var listener;
  this._once(type, listener = this._eventListener(type, pattern, callback));
  return listener;
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/lib/Model.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/lib/transaction.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/lib";
    var __filename = "/node_modules/racer/lib/transaction.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/lib/transaction.js"]._cached = module.exports;
    
    (function () {
        module.exports = {
  create: function(obj) {
    var txn;
    txn = [obj.base, obj.id, obj.method, obj.args];
    if (obj.meta) {
      txn.push(obj.meta);
    }
    return txn;
  },
  base: function(txn, val) {
    if (val !== void 0) {
      txn[0] = val;
    }
    return txn[0];
  },
  id: function(txn, val) {
    if (val !== void 0) {
      txn[1] = val;
    }
    return txn[1];
  },
  clientIdAndVer: function(txn) {
    var res;
    res = this.id(txn).split('.');
    res[1] = parseInt(res[1], 10);
    return res;
  },
  method: function(txn, name) {
    if (name !== void 0) {
      txn[2] = name;
    }
    return txn[2];
  },
  args: function(txn, vals) {
    if (vals !== void 0) {
      txn[3] = vals;
    }
    return txn[3];
  },
  path: function(txn, val) {
    var args;
    args = this.args(txn);
    if (val !== void 0) {
      args[0] = val;
    }
    return args[0];
  },
  clientId: function(txn, newClientId) {
    var clientId, num, _ref;
    _ref = this.id(txn).split('.'), clientId = _ref[0], num = _ref[1];
    if (newClientId !== void 0) {
      this.id(txn, newClientId + '.' + num);
      return newClientId;
    }
    return clientId;
  },
  meta: function(txn, obj) {
    if (obj !== void 0) {
      txn[4] = obj;
    }
    return txn[4];
  }
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/lib/transaction.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/lib/pathParser.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/lib";
    var __filename = "/node_modules/racer/lib/pathParser.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/lib/pathParser.js"]._cached = module.exports;
    
    (function () {
        module.exports = {
  isPrivate: function(name) {
    return /(^_)|(\._)/.test(name);
  },
  regExp: function(pattern) {
    if (pattern instanceof RegExp) {
      return pattern;
    } else {
      return new RegExp('^' + pattern.replace(/\.|\*{1,2}/g, function(match, index) {
        if (match === '.') {
          return '\\.';
        }
        if (match === '*') {
          return '([^\\.]+)';
        }
        if (match === '**') {
          if (pattern.length - index === 2) {
            return '(.+)';
          } else {
            return '(.+?)';
          }
        }
      }) + '$');
    }
  },
  fastLookup: function(path, obj) {
    var prop, _i, _len, _ref;
    _ref = path.split('.');
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      prop = _ref[_i];
      if (!(obj = obj[prop])) {
        return;
      }
    }
    return obj;
  }
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/lib/pathParser.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/lib/adapters/MemorySync.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/lib/adapters";
    var __filename = "/node_modules/racer/lib/adapters/MemorySync.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/lib/adapters");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/lib/adapters");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/lib/adapters/MemorySync.js"]._cached = module.exports;
    
    (function () {
        var Memory, merge, specHelper;
var __slice = Array.prototype.slice, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
merge = require('../util').merge;
specHelper = require('../specHelper');
Memory = module.exports = function() {
  this._data = {};
  this.ver = 0;
};
Memory.prototype = {
  get: function(path, obj) {
    if (obj == null) {
      obj = this._data;
    }
    if (path) {
      return this.lookup(path, false, {
        obj: obj
      }).obj;
    } else {
      return obj;
    }
  },
  set: function(path, value, ver, options) {
    var obj, out, parent, prop, refObjCopy, _ref;
    if (options == null) {
      options = {};
    }
    if (value && value.$r) {
      refObjCopy = merge({}, value);
      value = refObjCopy;
    }
    this.ver = ver;
    _ref = out = this.lookup(path, true, options), parent = _ref.parent, prop = _ref.prop;
    obj = out.obj = parent[prop] = value;
    if (options.returnMeta) {
      return out;
    } else {
      return obj;
    }
  },
  del: function(path, ver, options) {
    var curr, key, obj, out, parent, parentProto, prop, value, _ref;
    if (options == null) {
      options = {};
    }
    this.ver = ver;
    _ref = out = this.lookup(path, false, options), parent = _ref.parent, prop = _ref.prop, obj = _ref.obj, path = _ref.path;
    if (!parent) {
      if (options.returnMeta) {
        return out;
      } else {
        return obj;
      }
    }
    if (options.proto) {
      parentProto = Object.getPrototypeOf(parent);
      if (prop in parentProto) {
        curr = {};
        for (key in parentProto) {
          value = parentProto[key];
          if (key !== prop) {
            curr[key] = typeof value === 'object' ? Object.create(value) : value;
          }
        }
        parent.__proto__ = curr;
      }
    }
    delete parent[prop];
    if (options.returnMeta) {
      return out;
    } else {
      return obj;
    }
  },
  push: function() {
    var arr, options, out, path, ret, values, ver, _i;
    path = arguments[0], values = 4 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 2) : (_i = 1, []), ver = arguments[_i++], options = arguments[_i++];
    if (options === void 0) {
      options = {};
    }
    if (options.constructor !== Object) {
      values.push(ver);
      ver = options;
      options = {};
    }
    this.ver = ver;
    options.array = true;
    out = this.lookup(path, true, options);
    arr = out.obj;
    if (!specHelper.isArray(arr)) {
      throw new Error('Not an Array');
    }
    ret = arr.push.apply(arr, values);
    if (options.returnMeta) {
      return out;
    } else {
      return ret;
    }
  },
  pop: function(path, ver, options) {
    var arr, out, ret;
    if (options == null) {
      options = {};
    }
    this.ver = ver;
    options.array = true;
    out = this.lookup(path, true, options);
    arr = out.obj;
    if (!specHelper.isArray(arr)) {
      throw new Error('Not an Array');
    }
    ret = arr.pop();
    if (options.returnMeta) {
      return out;
    } else {
      return ret;
    }
  },
  insertAfter: function(path, afterIndex, value, ver, options) {
    var arr, out, ret;
    if (options == null) {
      options = {};
    }
    this.ver = ver;
    options.array = true;
    out = this.lookup(path, true, options);
    arr = out.obj;
    if (!specHelper.isArray(arr)) {
      throw new Error('Not an Array');
    }
    if (!((-1 <= afterIndex && afterIndex <= arr.length - 1))) {
      throw new Error('Out of Bounds');
    }
    ret = arr.splice(afterIndex + 1, 0, value);
    if (options.returnMeta) {
      return out;
    } else {
      return ret;
    }
  },
  insertBefore: function(path, beforeIndex, value, ver, options) {
    var arr, out, ret;
    if (options == null) {
      options = {};
    }
    this.ver = ver;
    options.array = true;
    out = this.lookup(path, true, options);
    arr = out.obj;
    if (!specHelper.isArray(arr)) {
      throw new Error('Not an Array');
    }
    if (!((0 <= beforeIndex && beforeIndex <= arr.length))) {
      throw new Error('Out of Bounds');
    }
    ret = arr.splice(beforeIndex, 0, value);
    if (options.returnMeta) {
      return out;
    } else {
      return ret;
    }
  },
  remove: function(path, startIndex, howMany, ver, options) {
    var arr, out, ret, upperBound;
    if (options == null) {
      options = {};
    }
    this.ver = ver;
    out = this.lookup(path, true, options);
    arr = out.obj;
    if (!specHelper.isArray(arr)) {
      throw new Error('Not an Array');
    }
    upperBound = arr.length ? arr.length - 1 : 0;
    if (!((0 <= startIndex && startIndex <= upperBound))) {
      throw new Error('Out of Bounds');
    }
    ret = arr.splice(startIndex, howMany);
    if (options.returnMeta) {
      return out;
    } else {
      return ret;
    }
  },
  splice: function() {
    var arr, newMembers, options, out, path, removeCount, ret, startIndex, ver, _i;
    path = arguments[0], startIndex = arguments[1], removeCount = arguments[2], newMembers = 6 <= arguments.length ? __slice.call(arguments, 3, _i = arguments.length - 2) : (_i = 3, []), ver = arguments[_i++], options = arguments[_i++];
    if (options === void 0) {
      options = {};
    }
    if (options.constructor !== Object) {
      newMembers.push(ver);
      ver = options;
      options = {};
    }
    this.ver = ver;
    options.array = true;
    out = this.lookup(path, true, options);
    arr = out.obj;
    if (!specHelper.isArray(arr)) {
      throw new Error('Not an Array');
    }
    ret = arr.splice.apply(arr, [startIndex, removeCount].concat(__slice.call(newMembers)));
    if (options.returnMeta) {
      return out;
    } else {
      return ret;
    }
  },
  unshift: function() {
    var arr, newMembers, options, out, path, ret, ver, _i;
    path = arguments[0], newMembers = 4 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 2) : (_i = 1, []), ver = arguments[_i++], options = arguments[_i++];
    if (options == null) {
      options = {};
    }
    if (options === void 0) {
      options = {};
    }
    if (options.constructor !== Object) {
      newMembers.push(ver);
      ver = options;
      options = {};
    }
    this.ver = ver;
    options.array = true;
    out = this.lookup(path, true, options);
    arr = out.obj;
    if (!specHelper.isArray(arr)) {
      throw new Error('Not an Array');
    }
    ret = arr.unshift.apply(arr, newMembers);
    if (options.returnMeta) {
      return out;
    } else {
      return ret;
    }
  },
  shift: function(path, ver, options) {
    var arr, out, ret;
    if (options == null) {
      options = {};
    }
    this.ver = ver;
    options.array = true;
    out = this.lookup(path, true, options);
    arr = out.obj;
    if (!specHelper.isArray(arr)) {
      throw new Error('Not an Array');
    }
    ret = arr.shift();
    if (options.returnMeta) {
      return out;
    } else {
      return ret;
    }
  },
  move: function(path, from, to, ver, options) {
    var value;
    if (options == null) {
      options = {};
    }
    value = this.lookup("" + path + "." + from, false, options).obj;
    if (to < 0) {
      to += this.lookup(path, false, options).obj.length;
    }
    if (from > to) {
      this.insertBefore(path, to, value, ver, options);
      from++;
    } else {
      this.insertAfter(path, to, value, ver, options);
    }
    return this.remove(path, from, 1, ver, options);
  },
  lookup: function(path, addPath, options) {
    var arrIndex, array, dereffedPath, i, key, keyVal, len, next, origPath, parent, pathSegment, prop, props, proto, rObj, rPath, rRemainingProps, ref, _ref;
    origPath = path;
    proto = options.proto, array = options.array;
    next = options.obj || this._data;
    props = path.split('.');
    path = '';
    i = 0;
    len = props.length;
    while (i < len) {
      parent = next;
      prop = props[i++];
      if (proto && !specHelper.isSpeculative(parent)) {
        parent = specHelper.create(parent);
      }
      pathSegment = prop.dereffedProp || prop;
      prop = prop.arrIndex !== void 0 ? prop.arrIndex : prop;
      path = path ? path + '.' + pathSegment : pathSegment;
      next = parent[prop];
      if (next === void 0) {
        if (!addPath) {
          return {
            obj: next,
            path: path,
            remainingProps: props.slice(i)
          };
        }
        if (array && i === len) {
          next = proto ? specHelper.create([]) : [];
        } else {
          next = proto ? specHelper.create({}) : {};
        }
        parent[prop] = next;
      } else if (proto && typeof next === 'object' && !specHelper.isSpeculative(next)) {
        if (specHelper.isSpeculative(parent) && parent.hasOwnProperty(prop)) {
          next._proto = true;
        } else {
          next = specHelper.create(next);
        }
        parent[prop] = next;
      }
      if (ref = next.$r) {
        if (i === len && options.dontFollowLastRef) {
          return {
            path: path,
            parent: parent,
            prop: prop,
            obj: next
          };
        }
        _ref = this.lookup(ref, addPath, options), rObj = _ref.obj, rPath = _ref.path, rRemainingProps = _ref.remainingProps;
        dereffedPath = rPath + ((rRemainingProps != null ? rRemainingProps.length : void 0) ? '.' + rRemainingProps.join('.') : '');
        if (!(key = next.$k)) {
          next = rObj;
        } else {
          keyVal = this.lookup(key, false, options).obj;
          if (specHelper.isArray(keyVal)) {
            next = keyVal.map(__bind(function(key) {
              return this.lookup(dereffedPath + '.' + key, false, options).obj;
            }, this));
            if (props[i]) {
              props[i] = {
                arrIndex: arrIndex = parseInt(props[i], 10),
                dereffedProp: keyVal[arrIndex]
              };
            }
          } else {
            dereffedPath = dereffedPath + '.' + this.lookup(key, false, options).obj;
            next = this.lookup(dereffedPath, addPath, options).obj;
          }
        }
        if (i < len) {
          path = dereffedPath;
        }
        if (next === void 0 && !addPath && i < len) {
          return {
            obj: next,
            path: path,
            remainingProps: props.slice(i)
          };
        }
      }
    }
    return {
      path: path,
      parent: parent,
      prop: prop,
      obj: next
    };
  }
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/lib/adapters/MemorySync.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/lib/specHelper.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/lib";
    var __filename = "/node_modules/racer/lib/specHelper.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/lib/specHelper.js"]._cached = module.exports;
    
    (function () {
        module.exports = {
  isArray: function(x) {
    return Array.isArray(x) || typeof x === 'object' && Array.isArray(Object.getPrototypeOf(x));
  },
  create: function(x) {
    var y;
    y = Object.create(x);
    y._proto = true;
    if (Array.isArray(x)) {
      y.toString = function() {
        var arr;
        arr = [];
        for (var i = 0, l = arr.length; i < l; i++) arr.push[i];;
        return arr.toString();
      };
    }
    return y;
  },
  reserved: ['_proto', 'toString'],
  isSpeculative: function(obj) {
    return '_proto' in obj;
  }
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/lib/specHelper.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/lib/TxnApplier.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/lib";
    var __filename = "/node_modules/racer/lib/TxnApplier.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/lib/TxnApplier.js"]._cached = module.exports;
    
    (function () {
        var DEFAULT_TIMEOUT, TxnApplier, transaction;
transaction = require('./transaction');
DEFAULT_TIMEOUT = 1000;
module.exports = TxnApplier = function(_arg) {
  var onTimeout, self, timeout;
  this.applyTxn = _arg.applyTxn, onTimeout = _arg.onTimeout, timeout = _arg.timeout;
  self = this;
  if (onTimeout) {
    if (timeout === void 0) {
      timeout = DEFAULT_TIMEOUT;
    }
    self._setWaiter = function() {
      if (this._waiter) {
        return;
      }
      return this._waiter = setTimeout(function() {
        onTimeout();
        return self._clearWaiter();
      }, timeout);
    };
    self._clearWaiter = function() {
      if (this._waiter) {
        clearTimeout(this._waiter);
        return this._waiter = null;
      }
    };
  }
  self._pending = {};
  self._index = 1;
};
TxnApplier.prototype = {
  _setWaiter: function() {},
  _clearWaiter: function() {},
  add: function(txn, txnIndex) {
    var index, pending;
    index = this._index;
    if (txnIndex > index) {
      this._pending[txnIndex] = txn;
      this._setWaiter();
      return true;
    }
    if (txnIndex < index) {
      return false;
    }
    this.applyTxn(txn, index);
    this._clearWaiter();
    index++;
    pending = this._pending;
    while (txn = pending[index]) {
      this.applyTxn(txn, index);
      delete pending[index++];
    }
    this._index = index;
    return true;
  },
  setIndex: function(_index) {
    this._index = _index;
  },
  clearPending: function() {
    var i, index, pending, _results;
    index = this._index;
    pending = this._pending;
    _results = [];
    for (i in pending) {
      _results.push(i < index ? delete pending[i] : void 0);
    }
    return _results;
  }
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/lib/TxnApplier.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/lib/RefHelper.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/lib";
    var __filename = "/node_modules/racer/lib/RefHelper.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/lib/RefHelper.js"]._cached = module.exports;
    
    (function () {
        var ARRAY_OPS, RefHelper, argsNormalizer, hasKeys, merge, pathParser, specHelper, transaction, _ref;
var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; }, __slice = Array.prototype.slice;
transaction = require('./transaction');
pathParser = require('./pathParser');
specHelper = require('./specHelper');
_ref = require('./util'), merge = _ref.merge, hasKeys = _ref.hasKeys;
argsNormalizer = require('./argsNormalizer');
module.exports = RefHelper = function(model) {
  this._model = model;
  this._adapter = model._adapter;
  this._setup();
};
ARRAY_OPS = {
  push: 1,
  unshift: 1,
  pop: 1,
  shift: 1,
  remove: 1,
  insertAfter: 1,
  insertBefore: 1,
  splice: 1,
  move: 1
};
RefHelper.prototype = {
  isPointer: function(val) {
    return !!(val && val.$r);
  },
  isKeyPath: function(path, obj) {
    var found;
    if (!obj) {
      throw new Error('Missing obj');
    }
    found = this._adapter.lookup("$keys." + path + ".$", false, {
      obj: obj
    }).obj;
    return found !== void 0;
  },
  isPathPointedTo: function(path, obj) {
    var found;
    if (!obj) {
      throw 'Missing obj';
    }
    found = this._adapter.lookup("$refs." + path + ".$", false, {
      obj: obj
    }).obj;
    return found !== void 0;
  },
  _setup: function() {
    var adapter, refHelper;
    refHelper = this;
    adapter = this._adapter;
    adapter.__set = adapter.set;
    adapter.set = function(path, value, ver, options) {
      var eachNode, maybeIndexRefs, out;
      if (options == null) {
        options = {};
      }
      out = null;
      options.obj || (options.obj = this._data);
      maybeIndexRefs = __bind(function(_path, _value) {
        if (refHelper.isPointer(_value)) {
          refHelper.$indexRefs(_path, _value.$r, _value.$k, _value.$t, ver, options);
        }
        if (_path === path) {
          out = this.__set(path, value, ver, options);
        }
        return refHelper.updateRefsForKey(_path, ver, options);
      }, this);
      if (Object !== (value != null ? value.constructor : void 0)) {
        maybeIndexRefs(path, value);
      } else {
        eachNode = function(path, value, callback) {
          var nodePath, prop, val, _results;
          callback(path, value);
          _results = [];
          for (prop in value) {
            val = value[prop];
            nodePath = "" + path + "." + prop;
            _results.push(Object === (val != null ? val.constructor : void 0) ? eachNode(nodePath, val, callback) : callback(nodePath, val));
          }
          return _results;
        };
        eachNode(path, value, function(nodePath, nodeValue) {
          return maybeIndexRefs(nodePath, nodeValue);
        });
      }
      return out;
    };
    adapter.__del = adapter.del;
    adapter.del = function(path, ver, options) {
      var out;
      if (options == null) {
        options = {};
      }
      options.obj || (options.obj = this._data);
      out = this.__del(path, ver, options);
      if (refHelper.isPathPointedTo(path, options.obj)) {
        refHelper.cleanupPointersTo(path, options);
      }
      return out;
    };
    adapter.__remove = adapter.remove;
    adapter.remove = function(path, start, howMany, ver, options) {
      var out, startIndex;
      if (options == null) {
        options = {};
      }
      options.obj || (options.obj = this._data);
      startIndex = refHelper.arrRefIndex(start, path, options.obj);
      out = this.__remove(path, startIndex, howMany, ver, options);
      refHelper.updateRefsForKey(path, ver, options);
      return out;
    };
    adapter.__move = adapter.move;
    adapter.move = function(path, from, to, ver, options) {
      var out, startIndex;
      if (options == null) {
        options = {};
      }
      options.obj || (options.obj = this._data);
      startIndex = refHelper.arrRefIndex(from, path, options.obj);
      out = this.__move(path, startIndex, to, ver, options);
      refHelper.updateRefsForKey(path, ver, options);
      return out;
    };
    ['push', 'unshift'].forEach(function(method) {
      adapter['__' + method] = adapter[method];
      return adapter[method] = function() {
        var options, out, path, values, ver, _ref2;
        _ref2 = argsNormalizer.adapter[method](arguments), path = _ref2[0], values = _ref2[1], ver = _ref2[2], options = _ref2[3];
        options.obj || (options.obj = this._data);
        out = this['__' + method].apply(this, [path].concat(__slice.call(values), [ver], [options]));
        refHelper.updateRefsForKey(path, ver, options);
        return out;
      };
    });
    ['pop', 'shift'].forEach(function(method) {
      adapter['__' + method] = adapter[method];
      return adapter[method] = function(path, ver, options) {
        var out;
        if (options == null) {
          options = {};
        }
        options.obj || (options.obj = this._data);
        out = this['__' + method](path, ver, options);
        refHelper.updateRefsForKey(path, ver, options);
        return out;
      };
    });
    ['insertAfter', 'insertBefore'].forEach(function(method) {
      adapter['__' + method] = adapter[method];
      return adapter[method] = function(path, pos, value, ver, options) {
        var index, out;
        if (options == null) {
          options = {};
        }
        options.obj || (options.obj = this._data);
        index = refHelper.arrRefIndex(pos, path, options.obj);
        out = this['__' + method](path, index, value, ver, options);
        refHelper.updateRefsForKey(path, ver, options);
        return out;
      };
    });
    adapter.__splice = adapter.splice;
    return adapter.splice = function() {
      var newMembers, options, out, path, removeCount, start, startIndex, ver, _ref2;
      _ref2 = argsNormalizer.adapter.splice(arguments), path = _ref2[0], start = _ref2[1], removeCount = _ref2[2], newMembers = _ref2[3], ver = _ref2[4], options = _ref2[5];
      options.obj || (options.obj = this._data);
      startIndex = refHelper.arrRefIndex(start, path, options.obj);
      out = this.__splice.apply(this, [path, startIndex, removeCount].concat(__slice.call(newMembers), [ver], [options]));
      refHelper.updateRefsForKey(path, ver, options);
      return out;
    };
  },
  arrRefIndex: function(start, path, obj) {
    var arr, i, mem, startIndex, _len;
    if ('number' === typeof start) {
      return start;
    }
    arr = this._adapter.lookup(path, true, {
      obj: obj
    }).obj;
    if (this.isArrayRef(path, obj)) {
      startIndex = arr.length;
      for (i = 0, _len = arr.length; i < _len; i++) {
        mem = arr[i];
        if (mem.id === start.id || parseInt(mem.id, 10) === parseInt(start.id, 10)) {
          return startIndex = i;
        }
      }
    }
    startIndex = arr.indexOf(start.id);
    if (startIndex !== -1) {
      return startIndex;
    }
    startIndex = arr.indexOf(parseInt(start.id, 10));
    if (startIndex !== -1) {
      return startIndex;
    }
    return arr.indexOf(start.id.toString());
  },
  ref: function(ref, key) {
    if (key != null) {
      return {
        $r: ref,
        $k: key
      };
    } else {
      return {
        $r: ref
      };
    }
  },
  arrayRef: function(ref, key) {
    return {
      $r: ref,
      $k: key,
      $t: 'array'
    };
  },
  $indexRefs: function(path, ref, key, type, ver, options) {
    var adapter, entry, keyOptions, keyVal, oldKey, oldRefObj, oldRefOptions, refHelper, refs, refsKey, refsKeys;
    adapter = this._adapter;
    refHelper = this;
    oldRefOptions = merge({
      dontFollowLastRef: true
    }, options);
    oldRefObj = adapter.lookup(path, false, oldRefOptions).obj;
    if (key) {
      entry = [ref, key];
      if (type) {
        entry.push(type);
      }
      adapter.lookup("$keys." + key + ".$", true, options).obj[path] = entry;
      keyVal = adapter.lookup(key, false, options).obj;
      if (type === void 0 && keyVal === void 0) {
        return;
      }
      if (type === 'array') {
        keyOptions = merge({
          array: true
        }, options);
        keyVal = adapter.lookup(key, true, keyOptions).obj;
        refsKeys = keyVal.map(function(keyValMem) {
          return ref + '.' + keyValMem;
        });
        this._removeOld$refs(oldRefObj, path, ver, options);
        return refsKeys.forEach(function(refsKey) {
          return refHelper._update$refs(refsKey, path, ref, key, type, options);
        });
      }
      refsKey = ref + '.' + keyVal;
    } else {
      if (oldRefObj && (oldKey = oldRefObj.$k)) {
        refs = adapter.lookup("$keys." + oldKey + ".$", false, options).obj;
        if (refs && refs[path]) {
          delete refs[path];
          if (!hasKeys(refs, {
            ignore: specHelper.reserved
          })) {
            adapter.del("$keys." + oldKey, ver, options);
          }
        }
      }
      refsKey = ref;
    }
    this._removeOld$refs(oldRefObj, path, ver, options);
    return this._update$refs(refsKey, path, ref, key, type, options);
  },
  _removeOld$refs: function(oldRefObj, path, ver, options) {
    var oldKey, oldKeyVal, oldRef, refHelper;
    if (oldRefObj && (oldRef = oldRefObj.$r)) {
      if (oldKey = oldRefObj.$k) {
        oldKeyVal = this._adapter.lookup(oldKey, false, options).obj;
      }
      if (oldKey && (oldRefObj.$t === 'array')) {
        refHelper = this;
        oldKeyVal.forEach(function(oldKeyMem) {
          return refHelper._removeFrom$refs(oldRef, oldKeyMem, path, ver, options);
        });
        return this._removeFrom$refs(oldRef, void 0, path, ver, options);
      } else {
        return this._removeFrom$refs(oldRef, oldKeyVal, path, ver, options);
      }
    }
  },
  _removeFrom$refs: function(ref, key, path, ver, options) {
    var refEntries, refWithKey;
    if (key) {
      refWithKey = ref + '.' + key;
    }
    refEntries = this._adapter.lookup("$refs." + refWithKey + ".$", false, options).obj;
    if (!refEntries) {
      return;
    }
    delete refEntries[path];
    if (!hasKeys(refEntries, {
      ignore: specHelper.reserved
    })) {
      return this._adapter.del("$refs." + ref, ver, options);
    }
  },
  _update$refs: function(refsKey, path, ref, key, type, options) {
    var entry, out;
    entry = [ref, key];
    if (type) {
      entry.push(type);
    }
    out = this._adapter.lookup("$refs." + refsKey + ".$", true, options);
    return out.obj[path] = entry;
  },
  updateRefsForKey: function(path, ver, options) {
    var refs;
    if (refs = this._adapter.lookup("$keys." + path + ".$", false, options).obj) {
      this._eachValidRef(refs, options.obj, __bind(function(path, ref, key, type) {
        return this.$indexRefs(path, ref, key, type, ver, options);
      }, this));
    }
    return this.eachValidRefPointingTo(path, options.obj, __bind(function(pointingPath, targetPathRemainder, ref, key, type) {
      return this.updateRefsForKey(pointingPath + '.' + targetPathRemainder, ver, options);
    }, this));
  },
  _eachValidRef: function(refs, obj, callback) {
    var fastLookup, key, o, path, ref, type, _ref2, _results;
    if (obj == null) {
      obj = this._adapter._data;
    }
    fastLookup = pathParser.fastLookup;
    _results = [];
    for (path in refs) {
      _ref2 = refs[path], ref = _ref2[0], key = _ref2[1], type = _ref2[2];
      if (path === '_proto') {
        continue;
      }
      o = fastLookup(path, obj);
      o = this._adapter.lookup(path, false, {
        obj: obj,
        dontFollowLastRef: true
      }).obj;
      _results.push(o && o.$r === ref && o.$k == key ? callback(path, ref, key, type) : delete refs[path]);
    }
    return _results;
  },
  _eachRefSetPointingTo: function(path, refs, fn) {
    var i, prop, props, refPos, refSet, _results;
    i = 0;
    refPos = refs;
    props = path.split('.');
    _results = [];
    while (prop = props[i++]) {
      if (!(refPos = refPos[prop])) {
        return;
      }
      _results.push((refSet = refPos.$) ? fn(refSet, props.slice(i).join('.'), prop) : void 0);
    }
    return _results;
  },
  eachValidRefPointingTo: function(targetPath, obj, fn) {
    var adapter, refs, self;
    self = this;
    adapter = self._adapter;
    if (!(refs = adapter.lookup('$refs', false, {
      obj: obj
    }).obj)) {
      return;
    }
    return self._eachRefSetPointingTo(targetPath, refs, function(refSet, targetPathRemainder, possibleIndex) {
      return self._eachValidRef(refSet, obj, function(pointingPath, ref, key, type) {
        if (type === 'array') {
          targetPathRemainder = possibleIndex + '.' + targetPathRemainder;
        }
        return fn(pointingPath, targetPathRemainder, ref, key, type);
      });
    });
  },
  eachArrayRefKeyedBy: function(path, obj, fn) {
    var key, ref, refSet, refs, type, _ref2, _results;
    if (!(refs = this._adapter.lookup("$keys", false, {
      obj: obj
    }).obj)) {
      return;
    }
    refSet = (path + '.$').split('.').reduce(function(refSet, prop) {
      return refSet && refSet[prop];
    }, refs);
    if (!refSet) {
      return;
    }
    _results = [];
    for (path in refSet) {
      _ref2 = refSet[path], ref = _ref2[0], key = _ref2[1], type = _ref2[2];
      _results.push(type === 'array' ? fn(path, ref, key) : void 0);
    }
    return _results;
  },
  notifyPointersTo: function(targetPath, obj, method, args, ignoreRoots) {
    if (ignoreRoots == null) {
      ignoreRoots = [];
    }
    this.eachValidRefPointingTo(targetPath, obj, __bind(function(pointingPath, targetPathRemainder, ref, key, type) {
      var id, index, rest, _ref2;
      if (type !== 'array') {
        if (this._alreadySeen(pointingPath, ref, ignoreRoots)) {
          return;
        }
        if (targetPathRemainder) {
          pointingPath += '.' + targetPathRemainder;
        }
      } else if (targetPathRemainder) {
        _ref2 = targetPathRemainder.split('.'), id = _ref2[0], rest = 2 <= _ref2.length ? __slice.call(_ref2, 1) : [];
        index = this._toIndex(key, id, obj);
        if (index !== -1) {
          pointingPath += '.' + index;
          if (rest.length) {
            pointingPath += '.' + rest.join('.');
          }
        }
      }
      return this._model.emit(method, [pointingPath].concat(__slice.call(args)));
    }, this));
    return this.eachArrayRefKeyedBy(targetPath, obj, __bind(function(pointingPath, ref, key) {
      var arrayMemberArgs, firstArgs, ns, _ref2;
      _ref2 = this.splitArrayArgs(method, args), firstArgs = _ref2[0], arrayMemberArgs = _ref2[1];
      if (arrayMemberArgs) {
        ns = this._adapter.lookup(ref, false, {
          obj: obj
        }).obj;
        arrayMemberArgs = arrayMemberArgs.map(function(arg) {
          return ns[arg] || ns[parseInt(arg, 10)];
        });
      }
      args = firstArgs.concat(arrayMemberArgs);
      return this._model.emit(method, [pointingPath].concat(__slice.call(args)));
    }, this));
  },
  _toIndex: function(arrayRefKey, id, obj) {
    var index, keyArr;
    keyArr = this._adapter.lookup(arrayRefKey, false, {
      array: true,
      obj: obj
    }).obj;
    index = keyArr.indexOf(id);
    if (index === -1) {
      return keyArr.indexOf(parseInt(id, 10));
    }
    return index;
  },
  _alreadySeen: function(pointingPath, ref, ignoreRoots) {
    var alreadySeen;
    alreadySeen = ignoreRoots.some(function(root) {
      return root === pointingPath.substr(0, root.length);
    });
    if (alreadySeen) {
      return true;
    }
    ignoreRoots.push(ref);
    return false;
  },
  cleanupPointersTo: function(path, options) {
    var adapter, key, keyMem, keyVal, model, pointingPath, ref, refs, _ref2, _results;
    adapter = this._adapter;
    refs = adapter.lookup("$refs." + path + ".$", false, options).obj;
    if (refs === void 0) {
      return;
    }
    model = this._model;
    _results = [];
    for (pointingPath in refs) {
      _ref2 = refs[pointingPath], ref = _ref2[0], key = _ref2[1];
      keyVal = key && adapter.lookup(key, false, options).obj;
      _results.push(keyVal && Array.isArray(keyVal) ? (keyMem = path.substr(ref.length + 1, pointingPath.length), adapter.remove(key, keyVal.indexOf(keyMem), 1, null, options)) : void 0);
    }
    return _results;
  },
  dereferenceTxn: function(txn, obj) {
    var $k, $r, args, method, newPushArgs, oldPushArgs, path, remainingProps, sliceFrom, _ref2, _ref3, _ref4;
    method = transaction.method(txn);
    args = transaction.args(txn);
    path = transaction.path(txn);
    if (ARRAY_OPS[method]) {
      sliceFrom = (function() {
        switch (method) {
          case 'push':
          case 'unshift':
            return 1;
          case 'pop':
          case 'shift':
          case 'insertAfter':
          case 'insertBefore':
            return 2;
          case 'remove':
          case 'move':
          case 'splice':
            return 3;
          default:
            throw new Error('Unimplemented for method ' + method);
        }
      })();
      if (_ref2 = this.isArrayRef(path, obj), $r = _ref2.$r, $k = _ref2.$k, _ref2) {
        this._model._cache.invalidateSpecModelCache();
        args[0] = path = $k;
        oldPushArgs = args.slice(sliceFrom);
        newPushArgs = oldPushArgs.map(function(refObjToAdd) {
          if (refObjToAdd.$r === void 0) {
            throw new Error('Trying to push a non-ref onto an array ref');
          }
          if ($r !== refObjToAdd.$r) {
            throw new Error("Trying to push elements of type " + refObjToAdd.$r + " onto path " + path + " that is an array ref of type " + $r);
          }
          return refObjToAdd.$k;
        });
        args.splice.apply(args, [sliceFrom, oldPushArgs.length].concat(__slice.call(newPushArgs)));
      } else {
        _ref3 = this._adapter.lookup(path, false, {
          obj: obj
        }), obj = _ref3.obj, path = _ref3.path, remainingProps = _ref3.remainingProps;
        if (obj === void 0 && (remainingProps != null ? remainingProps.length : void 0)) {
          args[0] = [path].concat(remainingProps).join('.');
        } else {
          args[0] = path;
        }
      }
      return txn;
    }
    _ref4 = this._adapter.lookup(path, false, {
      obj: obj
    }), obj = _ref4.obj, path = _ref4.path, remainingProps = _ref4.remainingProps;
    if (obj === void 0 && (remainingProps != null ? remainingProps.length : void 0)) {
      args[0] = [path].concat(remainingProps).join('.');
    } else {
      args[0] = path;
    }
    return txn;
  },
  isArrayRef: function(path, data) {
    var $k, $r, $t, options, refObj;
    options = {
      obj: data,
      dontFollowLastRef: true
    };
    refObj = this._adapter.lookup(path, false, options).obj;
    if (refObj === void 0) {
      return false;
    }
    $r = refObj.$r, $k = refObj.$k, $t = refObj.$t;
    if ($k) {
      $k = this.dereferencedPath($k, data);
    }
    if ($t === 'array') {
      return {
        $r: $r,
        $k: $k
      };
    } else {
      return false;
    }
  },
  splitArrayArgs: function(method, args) {
    switch (method) {
      case 'push':
        return [[], args];
      case 'insertAfter':
      case 'insertBefore':
        return [[args[0]], args.slice(1)];
      case 'splice':
        return [args.slice(0, 2), args.slice(2)];
      default:
        return [args, []];
    }
  },
  isRefObj: function(obj) {
    return '$r' in obj;
  },
  dereferencedPath: function(path, data) {
    var meta;
    meta = this._adapter.lookup(path, false, {
      obj: data,
      returnMeta: true
    });
    return meta.path;
  },
  denormalizePath: function(path, data) {
    var obj, remainingProps, _ref2;
    _ref2 = this._adapter.lookup(path, false, {
      obj: data
    }), path = _ref2.path, obj = _ref2.obj, remainingProps = _ref2.remainingProps;
    if (obj || !(remainingProps != null ? remainingProps.length : void 0)) {
      return path;
    }
    return [path].concat(remainingProps).join('.');
  }
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/lib/RefHelper.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/racer/lib/argsNormalizer.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/racer/lib";
    var __filename = "/node_modules/racer/lib/argsNormalizer.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/racer/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/racer/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/racer/lib/argsNormalizer.js"]._cached = module.exports;
    
    (function () {
        var argsNormalizer;
var __slice = Array.prototype.slice;
argsNormalizer = module.exports = {
  adapter: {
    push: function(_arguments) {
      var options, path, values, ver, _i, _ref;
      _ref = Array.prototype.slice.call(_arguments), path = _ref[0], values = 4 <= _ref.length ? __slice.call(_ref, 1, _i = _ref.length - 2) : (_i = 1, []), ver = _ref[_i++], options = _ref[_i++];
      if (options === void 0) {
        options = {};
      }
      if (options.constructor !== Object) {
        values.push(ver);
        ver = options;
        options = {};
      }
      return [path, values, ver, options];
    },
    splice: function(_arguments) {
      var newMembers, options, path, removeCount, startIndex, ver, _i, _ref;
      _ref = Array.prototype.slice.call(_arguments), path = _ref[0], startIndex = _ref[1], removeCount = _ref[2], newMembers = 6 <= _ref.length ? __slice.call(_ref, 3, _i = _ref.length - 2) : (_i = 3, []), ver = _ref[_i++], options = _ref[_i++];
      if (options === void 0) {
        options = {};
      }
      if (options.constructor !== Object) {
        newMembers.push(ver);
        ver = options;
        options = {};
      }
      return [path, startIndex, removeCount, newMembers, ver, options];
    }
  }
};
argsNormalizer.adapter.unshift = argsNormalizer.adapter.push;;
    }).call(module.exports);
    
    __require.modules["/node_modules/racer/lib/argsNormalizer.js"]._cached = module.exports;
    return module.exports;
};

require.modules["events"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = ".";
    var __filename = "events";
    
    var require = function (file) {
        return __require(file, ".");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, ".");
    };
    
    require.modules = __require.modules;
    __require.modules["events"]._cached = module.exports;
    
    (function () {
        if (!process.EventEmitter) process.EventEmitter = function () {};

var EventEmitter = exports.EventEmitter = process.EventEmitter;
var isArray = Array.isArray;

// By default EventEmitters will print a warning if more than
// 10 listeners are added to it. This is a useful default which
// helps finding memory leaks.
//
// Obviously not all Emitters should be limited to 10. This function allows
// that to be increased. Set to zero for unlimited.
var defaultMaxListeners = 10;
EventEmitter.prototype.setMaxListeners = function(n) {
  if (!this._events) this._events = {};
  this._events.maxListeners = n;
};


EventEmitter.prototype.emit = function(type) {
  // If there is no 'error' event listener then throw.
  if (type === 'error') {
    if (!this._events || !this._events.error ||
        (isArray(this._events.error) && !this._events.error.length))
    {
      if (arguments[1] instanceof Error) {
        throw arguments[1]; // Unhandled 'error' event
      } else {
        throw new Error("Uncaught, unspecified 'error' event.");
      }
      return false;
    }
  }

  if (!this._events) return false;
  var handler = this._events[type];
  if (!handler) return false;

  if (typeof handler == 'function') {
    switch (arguments.length) {
      // fast cases
      case 1:
        handler.call(this);
        break;
      case 2:
        handler.call(this, arguments[1]);
        break;
      case 3:
        handler.call(this, arguments[1], arguments[2]);
        break;
      // slower
      default:
        var args = Array.prototype.slice.call(arguments, 1);
        handler.apply(this, args);
    }
    return true;

  } else if (isArray(handler)) {
    var args = Array.prototype.slice.call(arguments, 1);

    var listeners = handler.slice();
    for (var i = 0, l = listeners.length; i < l; i++) {
      listeners[i].apply(this, args);
    }
    return true;

  } else {
    return false;
  }
};

// EventEmitter is defined in src/node_events.cc
// EventEmitter.prototype.emit() is also defined there.
EventEmitter.prototype.addListener = function(type, listener) {
  if ('function' !== typeof listener) {
    throw new Error('addListener only takes instances of Function');
  }

  if (!this._events) this._events = {};

  // To avoid recursion in the case that type == "newListeners"! Before
  // adding it to the listeners, first emit "newListeners".
  this.emit('newListener', type, listener);

  if (!this._events[type]) {
    // Optimize the case of one listener. Don't need the extra array object.
    this._events[type] = listener;
  } else if (isArray(this._events[type])) {

    // Check for listener leak
    if (!this._events[type].warned) {
      var m;
      if (this._events.maxListeners !== undefined) {
        m = this._events.maxListeners;
      } else {
        m = defaultMaxListeners;
      }

      if (m && m > 0 && this._events[type].length > m) {
        this._events[type].warned = true;
        console.error('(node) warning: possible EventEmitter memory ' +
                      'leak detected. %d listeners added. ' +
                      'Use emitter.setMaxListeners() to increase limit.',
                      this._events[type].length);
        console.trace();
      }
    }

    // If we've already got an array, just append.
    this._events[type].push(listener);
  } else {
    // Adding the second element, need to change to array.
    this._events[type] = [this._events[type], listener];
  }

  return this;
};

EventEmitter.prototype.on = EventEmitter.prototype.addListener;

EventEmitter.prototype.once = function(type, listener) {
  var self = this;
  self.on(type, function g() {
    self.removeListener(type, g);
    listener.apply(this, arguments);
  });

  return this;
};

EventEmitter.prototype.removeListener = function(type, listener) {
  if ('function' !== typeof listener) {
    throw new Error('removeListener only takes instances of Function');
  }

  // does not use listeners(), so no side effect of creating _events[type]
  if (!this._events || !this._events[type]) return this;

  var list = this._events[type];

  if (isArray(list)) {
    var i = list.indexOf(listener);
    if (i < 0) return this;
    list.splice(i, 1);
    if (list.length == 0)
      delete this._events[type];
  } else if (this._events[type] === listener) {
    delete this._events[type];
  }

  return this;
};

EventEmitter.prototype.removeAllListeners = function(type) {
  // does not use listeners(), so no side effect of creating _events[type]
  if (type && this._events && this._events[type]) this._events[type] = null;
  return this;
};

EventEmitter.prototype.listeners = function(type) {
  if (!this._events) this._events = {};
  if (!this._events[type]) this._events[type] = [];
  if (!isArray(this._events[type])) {
    this._events[type] = [this._events[type]];
  }
  return this._events[type];
};
;
    }).call(module.exports);
    
    __require.modules["events"]._cached = module.exports;
    return module.exports;
};

require.modules["/package.json"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/";
    var __filename = "/package.json";
    
    var require = function (file) {
        return __require(file, "/");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/");
    };
    
    require.modules = __require.modules;
    __require.modules["/package.json"]._cached = module.exports;
    
    (function () {
        module.exports = {"name":"derby-chat-example","description":"Demo chat application for Derby","version":"0.0.4","main":"./server.js","dependencies":{"derby":"0.0.4","express":">=2","stylus":">=0.15","connect-gzip":">=0.1"},"private":true};
    }).call(module.exports);
    
    __require.modules["/package.json"]._cached = module.exports;
    return module.exports;
};

require.modules["/chat.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/";
    var __filename = "/chat.js";
    
    var require = function (file) {
        return __require(file, "/");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/");
    };
    
    require.modules = __require.modules;
    __require.modules["/chat.js"]._cached = module.exports;
    
    (function () {
        var derby, model, ready, view, _ref;
derby = require('derby');
_ref = derby.createApp(module, exports), ready = _ref.ready, model = _ref.model, view = _ref.view;
view.make('Title', 'Chat ({{_session.numMessages}}) - {{_session.user.name}}');
view.make('info', "<div id=info>{{^connected}}\n  {{#canConnect}}\n    Offline{{#_showReconnect}} \n      &ndash; <a bind=click:connect>Reconnect</a>\n    {{/}}\n  {{^}}\n    Unable to reconnect &ndash; \n    <a bind=click:reload>Reload</a>\n  {{/}}\n{{/}}</div>");
view.make('message', "<li><img src=img/s.png class={{users.(userId).picClass}} alt=\"\">\n  <div class=message>\n    <p><b>{{users.(userId).name}}</b>\n    <p>{{comment}}\n  </div>", {
  After: function() {
    return $('messages').scrollTop = $('messageList').offsetHeight;
  }
});
ready(function() {
  model.set('_showReconnect', true);
  exports.connect = function() {
    model.set('_showReconnect', false);
    setTimeout((function() {
      return model.set('_showReconnect', true);
    }), 1000);
    return model.socket.socket.connect();
  };
  exports.reload = window.location.reload;
  model.on('push', '_room.messages', function() {
    return model.incr('_session.numMessages');
  });
  return exports.postMessage = function() {
    model.push('_room.messages', {
      userId: model.get('_session.userId'),
      comment: model.get('_session.newComment')
    });
    model.set('_session.newComment', '');
    return false;
  };
});;
    }).call(module.exports);
    
    __require.modules["/chat.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/derby/package.json"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/derby";
    var __filename = "/node_modules/derby/package.json";
    
    var require = function (file) {
        return __require(file, "/node_modules/derby");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/derby");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/derby/package.json"]._cached = module.exports;
    
    (function () {
        module.exports = {"name":"derby","description":"Framework for creating realtime applications with automatically synced models and functionally defined views. HTML and updates are rendered both on the server and on the client.","version":"0.0.4","main":"./lib/derby.js","browserify":{"main":"./lib/derby.browser.js"},"dependencies":{"racer":"0.0.11","uglify-js":">=1.0.6"},"devDependencies":{"expresso":">=0.7.9","should":">=0.2.1","coffee-script":">=1.1.1","express":">=2.4.2"},"engines":{"node":">=0.4.0"}};
    }).call(module.exports);
    
    __require.modules["/node_modules/derby/package.json"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/derby/lib/derby.browser.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/derby/lib";
    var __filename = "/node_modules/derby/lib/derby.browser.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/derby/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/derby/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/derby/lib/derby.browser.js"]._cached = module.exports;
    
    (function () {
        var Dom, View, modelHelper, racer;
racer = require('racer');
modelHelper = require('./modelHelper');
Dom = require('./Dom');
View = require('./View');
exports.createApp = function(appModule, appExports) {
  var dom, model, view;
  racer.util.merge(appExports, racer);
  appExports.view = view = new View(appExports);
  appExports.ready = function(fn) {
    return racer.onready = fn;
  };
  dom = view.dom = new Dom(model = view.model = racer.model);
  modelHelper.init(model, dom, view);
  appModule.exports = function(idCount, modelBundle, modelEvents, domEvents) {
    view._idCount = idCount;
    racer.init(modelBundle);
    model.__events.set(modelEvents);
    dom.init(domEvents);
    return appExports;
  };
  return appExports;
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/derby/lib/derby.browser.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/derby/lib/modelHelper.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/derby/lib";
    var __filename = "/node_modules/derby/lib/modelHelper.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/derby/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/derby/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/derby/lib/modelHelper.js"]._cached = module.exports;
    
    (function () {
        var EventDispatcher;
EventDispatcher = require('./EventDispatcher');
exports.init = function(model, dom, view) {
  var event, events, get, _fn, _i, _len, _ref;
  get = model.get;
  events = model.__events = new EventDispatcher({
    onTrigger: function(path, listener, value, options) {
      var id, method, oldPath, partial, property;
      oldPath = listener[0], id = listener[1], method = listener[2], property = listener[3], partial = listener[4];
      if (!(oldPath === path || get(oldPath) === get(path))) {
        return false;
      }
      if (options && options.method) {
        method = options.method;
      }
      if (partial) {
        value = view.get(partial, value);
      }
      return dom.update(id, method, property, value);
    },
    onBind: function(path, listener) {
      return listener.unshift(path);
    }
  });
  model.on('set', function(_arg) {
    var path, value;
    path = _arg[0], value = _arg[1];
    return events.trigger(path, value);
  });
  model.on('push', function(_arg) {
    var path, value;
    path = _arg[0], value = _arg[1];
    return events.trigger(path, value, {
      method: 'appendHtml'
    });
  });
  _ref = ['connected', 'canConnect'];
  _fn = function(event) {
    return model.on(event, function(value) {
      return events.trigger(event, value);
    });
  };
  for (_i = 0, _len = _ref.length; _i < _len; _i++) {
    event = _ref[_i];
    _fn(event);
  }
  return model;
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/derby/lib/modelHelper.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/derby/lib/EventDispatcher.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/derby/lib";
    var __filename = "/node_modules/derby/lib/EventDispatcher.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/derby/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/derby/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/derby/lib/EventDispatcher.js"]._cached = module.exports;
    
    (function () {
        var EventDispatcher, hasKeys, swapQuotes;
hasKeys = require('racer').util.hasKeys;
swapQuotes = function(s) {
  return s.replace(/['"]/g, function(match) {
    if (match === '"') {
      return "'";
    } else {
      return '"';
    }
  });
};
EventDispatcher = module.exports = function(options) {
  var empty;
  if (options == null) {
    options = {};
  }
  empty = function() {};
  this._onTrigger = options.onTrigger || empty;
  this._onBind = options.onBind || empty;
  this._onUnbind = options.onUnbind || empty;
  this._names = {};
};
EventDispatcher.prototype = {
  bind: function(name, listener) {
    var key, names, obj;
    if (this._onBind(name, listener) === false) {
      return;
    }
    names = this._names;
    key = listener == null ? 'null' : JSON.stringify(listener);
    obj = names[name] || {};
    obj[key] = true;
    return names[name] = obj;
  },
  unbind: function(name, listener) {
    var names, obj;
    if (this._onUnbind(name, listener) === false) {
      return;
    }
    names = this._names;
    if (!(obj = names[name])) {
      return;
    }
    delete obj[JSON.stringify(listener)];
    if (!hasKeys(obj)) {
      return delete names[name];
    }
  },
  trigger: function(name, value, options) {
    var count, key, listener, listeners, names, onTrigger;
    names = this._names;
    listeners = names[name];
    onTrigger = this._onTrigger;
    count = 0;
    for (key in listeners) {
      count++;
      listener = JSON.parse(key);
      if (onTrigger(name, listener, value, options) !== false) {
        continue;
      }
      delete listeners[key];
      count--;
    }
    if (count === 0) {
      return delete names[name];
    }
  },
  get: function() {
    var listener, listeners, name, names, out;
    names = this._names;
    out = {};
    for (name in names) {
      listeners = names[name];
      out[name] = (function() {
        var _results;
        _results = [];
        for (listener in listeners) {
          _results.push(swapQuotes(listener));
        }
        return _results;
      })();
    }
    return out;
  },
  set: function(n) {
    var listener, listeners, name, names, obj, _results;
    names = this._names;
    _results = [];
    for (name in n) {
      listeners = n[name];
      obj = names[name] = {};
      _results.push((function() {
        var _i, _len, _results2;
        _results2 = [];
        for (_i = 0, _len = listeners.length; _i < _len; _i++) {
          listener = listeners[_i];
          _results2.push(obj[swapQuotes(listener)] = true);
        }
        return _results2;
      })());
    }
    return _results;
  }
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/derby/lib/EventDispatcher.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/derby/lib/Dom.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/derby/lib";
    var __filename = "/node_modules/derby/lib/Dom.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/derby/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/derby/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/derby/lib/Dom.js"]._cached = module.exports;
    
    (function () {
        var Dom, EventDispatcher, View, addListener, doc, element, elements, emptyEl, getMethods, setMethods, win;
EventDispatcher = require('./EventDispatcher');
View = require('./View');
elements = {
  $win: win = typeof window === 'object' && window,
  $doc: doc = win.document
};
emptyEl = doc && doc.createElement('div');
element = function(id) {
  return elements[id] || (elements[id] = doc.getElementById(id));
};
getMethods = {
  attr: function(el, attr) {
    return el.getAttribute(attr);
  },
  prop: function(el, prop) {
    return el[prop];
  },
  html: function(el) {
    return el.innerHTML;
  }
};
setMethods = {
  attr: function(value, el, attr) {
    return el.setAttribute(attr, value);
  },
  prop: function(value, el, props) {
    var i, last, prop;
    if (Array.isArray(props)) {
      last = props.length - 1;
      i = 0;
      while (i < last) {
        el = el[props[i++]];
      }
      prop = props[last];
    } else {
      prop = props;
    }
    return el[prop] = value;
  },
  propPolite: function(value, el, prop) {
    if (el !== doc.activeElement) {
      return el[prop] = value;
    }
  },
  html: function(value, el, escape) {
    return el.innerHTML = escape ? View.htmlEscape(value) : value;
  },
  appendHtml: function(value, el) {
    var child, _results;
    emptyEl.innerHTML = value;
    _results = [];
    while (child = emptyEl.firstChild) {
      _results.push(el.appendChild(child));
    }
    return _results;
  }
};
addListener = function() {};
Dom = module.exports = function(model) {
  var events;
  this.events = events = new EventDispatcher({
    onBind: function(name) {
      if (!(name in events._names)) {
        return addListener(name);
      }
    },
    onTrigger: function(name, listener, targetId) {
      var callback, el, fn, id, method, path, property, silent, value;
      if (listener.length === 2) {
        console.log((callback = listener[0], id = listener[1], listener));
      } else {
        fn = listener[0], path = listener[1], id = listener[2], method = listener[3], property = listener[4], silent = listener[5];
      }
      if (id !== targetId) {
        return;
      }
      if (!(el = element(id))) {
        return false;
      }
      if (callback) {
        callback();
        return;
      }
      value = getMethods[method](el, property);
      return (silent ? model.silent : model)[fn](path, value);
    }
  });
};
Dom.prototype = {
  init: function(domEvents) {
    var domHandler, events, name, _results;
    events = this.events;
    domHandler = function(e) {
      var target;
      target = e.target || e.srcElement;
      if (target.nodeType === 3) {
        target = target.parentNode;
      }
      return events.trigger(e.type, target.id);
    };
    if (doc.addEventListener) {
      addListener = function(name) {
        return doc.addEventListener(name, domHandler, false);
      };
    } else if (doc.attachEvent) {
      addListener = function(name) {
        return doc.attachEvent('on' + name, function() {
          return domHandler(event);
        });
      };
    }
    events.set(domEvents);
    _results = [];
    for (name in events._names) {
      _results.push(addListener(name));
    }
    return _results;
  },
  update: function(id, method, property, value) {
    var el;
    if (!(el = element(id))) {
      return false;
    }
    return setMethods[method](value, el, property);
  }
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/derby/lib/Dom.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/derby/lib/View.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/derby/lib";
    var __filename = "/node_modules/derby/lib/View.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/derby/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/derby/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/derby/lib/View.js"]._cached = module.exports;
    
    (function () {
        var View, addId, addNameToData, dataValue, elementParser, extend, extractPlaceholder, htmlEscape, htmlParser, modelPath, modelText, parse, parseString, quoteAttr, reduceStack, renderer, trim, trimInner;
htmlParser = require('./htmlParser');
elementParser = require('./elementParser');
View = module.exports = function(_exports) {
  var self;
  this._exports = _exports;
  self = this;
  this._views = {};
  this._loadFuncs = '';
  this._idCount = 0;
  this._uniqueId = function() {
    return '$' + (self._idCount++).toString(36);
  };
};
View.prototype = {
  get: function(viewName, ctx, parentCtx) {
    var i, item, parentView, type, view;
    if (!(view = this._views[viewName])) {
      if (~(i = viewName.indexOf('$'))) {
        parentView = viewName.substr(1, i - 1);
        if (!this._views[parentView]) {
          throw "Can't find view: " + parentView;
        }
        this.get(parentView);
        return this.get(viewName, ctx, parentCtx);
      }
      return '';
    }
    if (parentCtx || (ctx != null)) {
      if (Array.isArray(ctx)) {
        return ((function() {
          var _i, _len, _results;
          _results = [];
          for (_i = 0, _len = ctx.length; _i < _len; _i++) {
            item = ctx[_i];
            _results.push(view(extend(parentCtx, item)));
          }
          return _results;
        })()).join('');
      }
      ctx = (type = viewName.charAt(0)) === '^' ? !ctx : type === '#' ? !!ctx : ctx;
      if (ctx === false) {
        return '';
      }
    }
    return view(extend(parentCtx, ctx));
  },
  preLoad: function(fn) {
    return this._loadFuncs += "(" + fn + ")();";
  },
  make: function(name, template, data) {
    var fn, render, self;
    if (data == null) {
      data = {};
    }
    self = this;
    render = function(ctx) {
      if (typeof template === 'string' && !~template.indexOf('{{')) {
        template = trim(template);
        render = function() {
          return template;
        };
      } else {
        render = parse(self, name, template, data);
      }
      return render(ctx);
    };
    fn = function(ctx) {
      return render(extend(data, ctx));
    };
    return this._register(name, fn, data.Before, data.After);
  },
  _register: function(name, fn, before, after) {
    return this._views[name] = before ? after ? function(ctx) {
      var rendered;
      before();
      setTimeout(after, 0);
      return rendered = fn(ctx);
    } : function(ctx) {
      before();
      return fn(ctx);
    } : after ? function(ctx) {
      var rendered;
      setTimeout(after, 0);
      return rendered = fn(ctx);
    } : fn;
  }
};
extend = function(parent, obj) {
  var key, out;
  if (typeof parent !== 'object') {
    return obj;
  }
  if (!obj) {
    return parent;
  }
  out = Object.create(parent);
  for (key in obj) {
    out[key] = obj[key];
  }
  return out;
};
addId = function(attrs, uniqueId) {
  if (attrs.id === void 0) {
    return attrs.id = function() {
      return attrs._id = uniqueId();
    };
  }
};
View.htmlEscape = htmlEscape = function(s) {
  if (s == null) {
    return '';
  } else {
    return s.replace(/[&<>]/g, function(s) {
      switch (s) {
        case '&':
          return '&amp;';
        case '<':
          return '&lt;';
        case '>':
          return '&gt;';
        default:
          return s;
      }
    });
  }
};
quoteAttr = function(s) {
  if (s == null || s === '') {
    return '""';
  }
  s = s.toString().replace(/"/g, '&quot;');
  if (/[ =]/.test(s)) {
    return '"' + s + '"';
  } else {
    return s;
  }
};
trim = function(s) {
  if (s) {
    return s.replace(/(?:^|\n)\s*/g, '');
  } else {
    return '';
  }
};
trimInner = function(s) {
  if (s) {
    return s.replace(/\n\s*/g, '');
  } else {
    return '';
  }
};
extractPlaceholder = function(text) {
  var content, match;
  match = /^([^\{]*)(\{{2,3})([^\}]+)\}{2,3}([\s\S]*)/.exec(text);
  if (!match) {
    return;
  }
  content = /^([#^//]?) *([^ >]*)(?: *> *(.*))?/.exec(match[3]);
  return {
    pre: trimInner(match[1]),
    escaped: match[2] === '{{',
    type: content[1],
    name: content[2],
    partial: content[3],
    post: trimInner(match[4])
  };
};
addNameToData = function(data, name) {
  if (name && !(name in data)) {
    return data[name] = {
      model: name
    };
  }
};
modelPath = function(data, name) {
  var datum, path;
  datum = data[name];
  if (!(datum && (path = datum.model))) {
    return null;
  }
  return path.replace(/\(([^)]+)\)/g, function(match, name) {
    return data[name];
  });
};
dataValue = function(data, name, model) {
  var path, value;
  if (path = modelPath(data, name)) {
    if ((value = model.get(path)) != null) {
      return value;
    } else {
      return model[path];
    }
  } else {
    return data[name];
  }
};
modelText = function(view, name, escaped, quote) {
  return function(data, model) {
    var text;
    text = dataValue(data, name, model);
    text = text != null ? text.toString() : '';
    if (escaped) {
      text = htmlEscape(text);
    }
    if (quote) {
      text = quoteAttr(text);
    }
    return text;
  };
};
reduceStack = function(stack) {
  var html, i, item, key, pushValue, value, _i, _len, _ref;
  html = [''];
  i = 0;
  for (_i = 0, _len = stack.length; _i < _len; _i++) {
    item = stack[_i];
    pushValue = function(value, quote) {
      if (value && value.call) {
        return i = html.push(value, '') - 1;
      } else {
        return html[i] += quote ? quoteAttr(value) : value;
      }
    };
    switch (item[0]) {
      case 'start':
        html[i] += '<' + item[1];
        _ref = item[2];
        for (key in _ref) {
          value = _ref[key];
          html[i] += ' ' + key + '=';
          pushValue(value, true);
        }
        html[i] += '>';
        break;
      case 'chars':
        pushValue(item[1]);
        break;
      case 'end':
        html[i] += '</' + item[1] + '>';
    }
  }
  return html;
};
renderer = function(view, items, events) {
  return function(data) {
    var domEvents, event, html, item, model, modelEvents, _i, _len;
    model = view.model;
    modelEvents = model.__events;
    domEvents = view.dom.events;
    html = ((function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = items.length; _i < _len; _i++) {
        item = items[_i];
        _results.push((item.call ? item(data, model) : item));
      }
      return _results;
    })()).join('');
    for (_i = 0, _len = events.length; _i < _len; _i++) {
      event = events[_i];
      event(data, modelEvents, domEvents);
    }
    return html;
  };
};
parse = function(view, viewName, template, data) {
  var block, chars, elementBind, elementPlaceholder, events, partialCount, partialName, popped, queue, queues, stack, uniqueId, _fn, _i, _len, _ref;
  if (viewName === 'Title') {
    return parseString(view, viewName, template, data);
  }
  queues = [
    {
      stack: stack = [],
      events: events = []
    }
  ];
  popped = [];
  block = null;
  uniqueId = view._uniqueId;
  partialCount = 0;
  partialName = function() {
    return viewName + '$' + partialCount++;
  };
  _ref = elementParser(view, events), elementPlaceholder = _ref.elementPlaceholder, elementBind = _ref.elementBind;
  htmlParser.parse(template, {
    start: function(tag, tagName, attrs) {
      var attr, escaped, fn, match, method, name, value;
      for (attr in attrs) {
        value = attrs[attr];
        if (match = extractPlaceholder(value)) {
          name = match.name, escaped = match.escaped;
          addNameToData(data, name);
          addId(attrs, uniqueId);
          method = tagName in elementPlaceholder ? elementPlaceholder[tagName](attr, attrs, name) : 'attr';
          events.push(function(data, modelEvents) {
            var path;
            path = modelPath(data, name);
            if (path) {
              return modelEvents.bind(path, [attrs._id || attrs.id, method, attr]);
            }
          });
          attrs[attr] = modelText(view, name, escaped, true);
        }
        if (attr === 'bind') {
          if (!(match = /^\s*([^:\s]+)\s*:\s*([^\s]+)/.exec(value))) {
            break;
          }
          name = match[1];
          fn = match[2];
          delete attrs[attr];
          addId(attrs, uniqueId);
          if (tagName in elementBind) {
            elementBind[tagName](attrs, name);
          }
          events.push(function(data, modelEvents, domEvents) {
            return domEvents.bind(name, [view._exports[fn], attrs._id || attrs.id]);
          });
        }
      }
      return stack.push(['start', tagName, attrs]);
    },
    chars: chars = function(text) {
      var addEvent, attrs, autoClosed, endBlock, escaped, i, last, lastAutoClosed, lastPartial, match, name, partial, partialText, post, pre, startBlock, type, wrap, _ref2;
      if (match = extractPlaceholder(text)) {
        pre = match.pre, post = match.post, name = match.name, escaped = match.escaped, type = match.type, partial = match.partial;
        addNameToData(data, name);
        text = '';
      } else {
        text = trimInner(text);
      }
      if (pre) {
        stack.push(['chars', pre]);
      }
      if (block && (type === '/' || (autoClosed = type === '^' && !name && block.type === '#'))) {
        name = block.name, partial = block.partial, lastPartial = block.lastPartial, lastAutoClosed = block.lastAutoClosed;
        popped.push(queues.pop());
        _ref2 = queues[queues.length - 1], stack = _ref2.stack, events = _ref2.events, block = _ref2.block;
      }
      if (startBlock = type === '#' || type === '^') {
        lastPartial = partial;
        partial = type + partialName();
        block = {
          type: type,
          name: name,
          partial: partial,
          lastPartial: lastPartial,
          lastAutoClosed: autoClosed
        };
      }
      if (partial) {
        partialText = function(data, model) {
          return view.get(partial, dataValue(data, name, model), data);
        };
      }
      if (name && !startBlock) {
        endBlock = type === '/';
        i = stack.length - (endBlock ? (lastAutoClosed ? 3 : 2) : 1);
        last = stack[i];
        if (wrap = pre || post || !(last && last[0] === 'start')) {
          last = ['start', 'ins', {}];
          if (endBlock) {
            stack.splice(i + 1, 0, last);
          } else {
            stack.push(last);
          }
        }
        attrs = last[2];
        addId(attrs, uniqueId);
        addEvent = function(partial, append) {
          return events.push(function(data, modelEvents) {
            var domMethod, params, path;
            if (!(path = modelPath(data, name))) {
              return;
            }
            if (partial) {
              escaped = false;
            }
            domMethod = append ? 'appendHtml' : 'html';
            params = [attrs._id || attrs.id, domMethod, +escaped];
            if (partial) {
              params[3] = partial;
            }
            return modelEvents.bind(path, params);
          });
        };
        addEvent(partial);
        if (lastAutoClosed) {
          addEvent(lastPartial, true);
        }
        if (!endBlock) {
          text = partialText || modelText(view, name, escaped);
        }
      } else {
        text = partialText || text;
      }
      if (text) {
        stack.push(['chars', text]);
      }
      if (wrap) {
        stack.push(['end', 'ins']);
      }
      if (startBlock) {
        queues.push({
          stack: stack = [],
          events: events = [],
          viewName: partial,
          block: block
        });
      }
      if (post) {
        return chars(post);
      }
    },
    end: function(tag, tagName) {
      return stack.push(['end', tagName]);
    }
  });
  _fn = function(queue) {
    var render;
    render = renderer(view, reduceStack(queue.stack), queue.events);
    return view._register(queue.viewName, function(ctx) {
      return render(extend(data, ctx));
    });
  };
  for (_i = 0, _len = popped.length; _i < _len; _i++) {
    queue = popped[_i];
    _fn(queue);
  }
  return renderer(view, reduceStack(stack), events);
};
parseString = function(view, viewName, template, data) {
  var events, items, name, params, post, pre, _ref;
  items = [];
  events = [];
  post = template;
  while (post) {
    _ref = extractPlaceholder(post), name = _ref.name, pre = _ref.pre, post = _ref.post;
    addNameToData(data, name);
    if (pre) {
      items.push(pre);
    }
    items.push(modelText(view, name));
    if (viewName === 'Title') {
      params = ['$doc', 'prop', 'title', 'Title'];
    }
    (function(name) {
      if (params) {
        return events.push(function(data, modelEvents) {
          var path;
          if (!(path = modelPath(data, name))) {
            return;
          }
          return modelEvents.bind(path, params.slice());
        });
      }
    })(name);
  }
  return renderer(view, items, events);
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/derby/lib/View.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/derby/lib/htmlParser.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/derby/lib";
    var __filename = "/node_modules/derby/lib/htmlParser.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/derby/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/derby/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/derby/lib/htmlParser.js"]._cached = module.exports;
    
    (function () {
        var attr, comment, endTag, startTag;
startTag = /^<(\w+)((?:\s+\w+(?:\s*=\s*(?:(?:"[^"]*")|(?:'[^']*')|[^>\s]+)?)?)*)\s*(\/?)>/;
endTag = /^<\/(\w+)[^>]*>/;
attr = /(\w+)(?:\s*(=)\s*(?:(?:"((?:\\.|[^"])*)")|(?:'((?:\\.|[^'])*)')|([^>\s]+))?)?/g;
comment = /<!--[\s\S]*?-->/g;
exports.parse = function(html, handler) {
  var chars, charsHandler, empty, endHandler, index, last, match, parseEndTag, parseStartTag, startHandler, text, _results;
  parseStartTag = function(tag, tagName, rest) {
    var attrs;
    attrs = {};
    rest.replace(attr, function(match, name, equals, attr0, attr1, attr2) {
      return attrs[name.toLowerCase()] = attr0 || attr1 || attr2 || (equals ? '' : null);
    });
    return startHandler(tag, tagName.toLowerCase(), attrs);
  };
  parseEndTag = function(tag, tagName) {
    return endHandler(tag, tagName.toLowerCase());
  };
  empty = function() {};
  charsHandler = (handler && handler.chars) || empty;
  startHandler = (handler && handler.start) || empty;
  endHandler = (handler && handler.end) || empty;
  html = html.replace(comment, '');
  _results = [];
  while (html) {
    last = html;
    chars = true;
    if (html[0] === '<') {
      if (html[1] === '/') {
        match = html.match(endTag);
        if (match) {
          html = html.substring(match[0].length);
          match[0].replace(endTag, parseEndTag);
          chars = false;
        }
      } else {
        match = html.match(startTag);
        if (match) {
          html = html.substring(match[0].length);
          match[0].replace(startTag, parseStartTag);
          chars = false;
        }
      }
    }
    if (chars) {
      index = html.indexOf('<');
      text = index < 0 ? html : html.substring(0, index);
      html = index < 0 ? '' : html.substring(index);
      if (text) {
        charsHandler(text);
      }
    }
    _results.push((function() {
      if (html === last) {
        throw 'Parse error: ' + html;
      }
    })());
  }
  return _results;
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/derby/lib/htmlParser.js"]._cached = module.exports;
    return module.exports;
};

require.modules["/node_modules/derby/lib/elementParser.js"] = function () {
    var module = { exports : {} };
    var exports = module.exports;
    var __dirname = "/node_modules/derby/lib";
    var __filename = "/node_modules/derby/lib/elementParser.js";
    
    var require = function (file) {
        return __require(file, "/node_modules/derby/lib");
    };
    
    require.resolve = function (file) {
        return __require.resolve(name, "/node_modules/derby/lib");
    };
    
    require.modules = __require.modules;
    __require.modules["/node_modules/derby/lib/elementParser.js"]._cached = module.exports;
    
    (function () {
        module.exports = function(view, events) {
  return {
    elementPlaceholder: {
      input: function(attr, attrs, name) {
        var method, silent;
        if (attr !== 'value') {
          return 'attr';
        }
        if ('silent' in attrs) {
          delete attrs.silent;
          method = 'prop';
          silent = 1;
        } else {
          method = 'propPolite';
          silent = 0;
        }
        events.push(function(data, modelEvents, domEvents) {
          var domArgs;
          domArgs = ['set', data[name].model, attrs._id || attrs.id, 'prop', 'value', silent];
          domEvents.bind('keyup', domArgs);
          return domEvents.bind('keydown', domArgs);
        });
        return method;
      }
    },
    elementBind: {
      a: function(attrs, name) {
        if (name === 'click') {
          if (!attrs.href) {
            attrs.href = '#';
          }
          if (!attrs.onclick) {
            return attrs.onclick = 'return false';
          }
        }
      }
    }
  };
};;
    }).call(module.exports);
    
    __require.modules["/node_modules/derby/lib/elementParser.js"]._cached = module.exports;
    return module.exports;
};
